; Beta 1.7 (jpa,  8/apr/04) jasona@noao.edu
;      *Special case for immobile W2 cart is turned off
;      *Special case for Polaris added (show full 24 hours)
;      *plots now show only those points where target has an altitude
;       > 30 degrees, instead of the full 12 hour track.  X and Y ranges
;       on plot modified to show relavant hour angles
;      *now 10 zero contours in U,V plane plot
;  
; Beta 1.6 (jpa, 23/mar/04) jasona@noao.edu
;      *Theo found major sign-error bug in how the POP delays are summed.
;       This is now fixed!
; Beta 1.5  (jpa, 19/mar/04) jasona@noao.edu
;      *fix bug in Position Angle vs. Hour Angle legend.     
;      *added 'Make PS File' button to GUI. The jplotopen.pro
;       and jplotclose.pro routines have been added.
; Beta 1.4  (jpa, 16/mar/04) jasona@noao.edu
;      *HD number and angular diameter now entered by user, no more long lists
;      *More info added to Intro/Help window
;      *fixed bug in default POP setup
; Beta 1.3  (jpa, 2/mar/04) jasona@noao.edu
;      *latest baseline solution from Theo
; Beta 1.2  (jpa, 2/mar/04) jasona@noao.edu
;     *fixed bug in astronomical twilight computation 
;      (zenith distance of sun should be 108 deg not 120 deg!)
;
; Beta 1.1  (jpa, 1/mar/04) jasona@noao.edu
;    *X-Y-Z components of telescopes used is U,V 
;        computations
;    *Added POP converage constraints to UV-plane plot
;    *Tested delay coverage plot against Theo's 
;          "chara_plan" on pease.chara-array.org
;           For HD 197345 on S1/POP1 - E1/POP4 the required delay drops
;           below -42m in Theo's program, not this one.  
;           Lack of Z-coordinate telescope data was the problem.
;           "Delay possible" calculations with XYZ components
;           agree within ~5 minutes;
;    *Separated "introduction/help" window from "output window"
;
; Beta 1.0  (jpa, 28/feb/04) jasona@noao.edu
;    initial pre-release version (thanks to John Monnier for trying it out!)
;
;**************************************************************************
;  CCCCC H   H  AAA   AAAA    AAA     PPPP   LL       AAA   N    N
;  C     H   H A   A  A   A  A   A    P   P  LL      A   A  N N  N
;  C     HHHHH AAAAA  AAAA   AAAAA    PPPP   LL      AAAAA  N  N N
;  C     H   H A   A  A   A  A   A    P      LL      A   A  N   NN 
;  CCCCC H   H A   A  A   A  A   A    P      LLLLLL  A   A  N    N
;***************************************************************************
; *PURPOSE:  A GUI interface to the chara_config routine for
;            planning observations with the CHARA inteferometric array
;
;*****************************************
;*****************************************
; ***To run this IDL GUI program***
;*****************************************
; first compile it:
;
; IDL> .run chara_plan
; % Compiled module: PCHECK.
; % Compiled module: IUEGETOPT.
; % Compiled module: PARCHECK.
; % Compiled module: TEN.
; % Compiled module: CHARA_EVENT.
; % Compiled module: CHARA_PLAN.
; % Compiled module: CHARA_CONFIG.
; % Compiled module: SUNPOS.
; % Compiled module: ORBIT2.
; % Compiled module: MYKEPLER.
; % Compiled module: JULDATE.
; % Compiled module: LEGEND.
;
; then type
;
; IDL> chara_plan     
;*****************************************
;*****************************************
;
; ***what do these modules do?*****
;
;  CHARA_CONFIG.     - does the visibility, delay calculations & plotting
;      JULDATE.      - julian date from calendar date (from IUE library)
;         PCHECK.    - dependancy of juldate (from IUE library)
;         IUEGETOPT. - dependancy of juldate (from IUE library)
;         PARCHECK.  - dependancy of juldate (from IUE library)
;      SUNPOS.       - compute position of the sun 
;         TEN.       - dependancy of sunpos (from ASTRO library)
;      ORBIT2.       - compute position angle and separtion of binary alpha Vir
;        MYKEPLER.   - compute true anomaly (solve Kepler's equation) for an orbit
;      LEGEND        - make plot legends (from IUE library) 
; CHARA_PLAN.        - build the chara_plan GUI widget 
; CHARA_EVENT.       - manage the GUI
;
;***************************************************************************


;************************************************************************
;+
;
;*NAME:
;
;    PCHECK
;
;*CLASS:
;
;*CATEGORY:
;
;*PURPOSE:
;
;    This procedure is used to check the parameters of
;    a procedure for correct type and dimensions
;
;*CALLING SEQUENCE:
;
;    PCHECK,VARIABLE,POSITION,DIMENSION,TYPE
;
;*PARAMETERS:
;
;    VARIABLE  - Variable to be checked
;    POSITION  - Parameter position of the variable
;    DIMENSION - Valid dimensions (3 digit integer).	
; 	    	 Each digit must be 0 or 1.
;		 1 specifies that the dimension is valid.
;
;		   1st digit  -   scalar
;		   2nd        -   one dimensional array
;		   3rd        -   two dimensional array
;
;    TYPE      - Valid types ( 4 digit integer).
;		 Each digit must be 0 or 1.
;		 1 specifies a valid type.
;
;		   1st digit -  string
;		   2nd       -  byte
;		   3rd       -  integer, or longword integer
;		   4th       -  floating point, double precision or complex
;
;*EXAMPLES:
;
;*SYSTEM VARIABLES USED:
;
;*INTERACTIVE INPUT:
;
;*SUBROUTINES CALLED:
;
;*FILES USED:
;
;*SIDE EFFECTS:
;
;*RESTRICTIONS:
;    
;    Only handles scalar, vector, or 2-d array parameters. pcheck returns
;    if the input variable has more than 2 dimensions.
;
;*NOTES:
;
;    tested with IDL Version 2.1.0  (sunos sparc)    28 Jun 91
;    tested with IDL Version 2.1.0  (ultrix mipsel)  N/A
;    tested with IDL Version 2.1.0  (vax vms)        28 Jun 91
;
;*PROCEDURE:
;
;*I_HELP  nn:
;
;*MODIFICATION HISTORY:
;
;    d. lindler dec. 1980
;    version 2 f.h.schiffer 3rd 17 nov 1981
;            copied into [177001] by rwt 1-17-84
;    10-24-85 rwt modify for new didl data types: longword integer,
;             double precision & complex. modified to be compatible
;             with existing version (i.e., no new input parameters)
;    21-sep-88:  converted to sun idl, john hoegy.
;    10 May 91	PJL created a RDAF format prolog
;    28 Jun 91  PJL added npar test; tested on SUN and VAX; updated prolog
;    10 Jun 94  RWT comment out last print message and STOP command, make 
;             position a byte value, line up print messages, and add comment 
;             about dimension restrictions.
;-
;*************************************************************************
 pro pcheck,variable,position,dimension,type
;
 npar = n_params(0)
 if npar eq 0 then begin
    print,'PCHECK,VARIABLE,POSITION,DIMENSION,TYPE'
    retall
 endif  ; npar
;
; decode valid dimensions
;
 vdim = bytarr(3)
 idim = dimension
 position = byte(position)
 for i = 0,2 do begin
    itemp = idim / 10*10
    vdim(2 - i) = idim - itemp
    idim = idim / 10
 endfor  ; i
;
; decode valid types
;
 itype = fix(type)
 vtype = bytarr(4)
 for i = 0,3 do begin
    itemp = itype / 10*10
    vtype(i) = itype - itemp
    itype = itype / 10
 endfor  ; i
;
; determine type and dimension of the input variable
;
 s = size(variable)
 ndim = s(0)
 if (ndim ge 3) then begin
    print,' Input variable in position',position,' has more than 2 dimensions'
    print,' Returning from PCHECK'
    retall 
 endif
 ntype = s(ndim + 1)
;
; check if it is defined
;
 if ntype eq 0 then begin
    print,' Input variable number',position,' is not defined'
    goto,explain
 endif  ; ntype eq 0
;
; check for valid type
;
 if (ntype eq 7) and ( vtype(3) ne 0 ) then goto,checkd ;string
 if (ntype eq 1) and ( vtype(2) ne 0 ) then goto,checkd ;byte
 if (ntype eq 2) and ( vtype(1) ne 0 ) then goto,checkd ;integer
 if (ntype eq 3) and ( vtype(1) ne 0 ) then goto,checkd ;longword integer
 if (ntype eq 4) and ( vtype(0) ne 0 ) then goto,checkd ;floating pt.
 if (ntype eq 5) and ( vtype(0) ne 0 ) then goto,checkd ;double precision
 if (ntype eq 6) and ( vtype(0) ne 0 ) then goto,checkd ;complex
;
 print,' The parameter in position',position,' is of invalid type'
 goto,explain
;
;
; check for a valid dimension
;
 checkd:  if vdim(ndim) ne 0 then return
 print,' Input variable in position',position,' has wrong dimension'
;
; give correct type(s) and dimensions
;
 explain:
 tmsg='    The valid type(s) are: '
 if vtype(3) ne 0 then tmsg = tmsg + ' string '
 if vtype(2) ne 0 then tmsg = tmsg + ' byte '
 if vtype(1) ne 0 then tmsg = tmsg + ' integer '
 if vtype(0) ne 0 then tmsg = tmsg + ' floating point '
 print,tmsg
;
 tmsg='    The valid dimensions are:'
 if vdim(2) ne 0 then tmsg = tmsg + ' 2-d '
 if vdim(1) ne 0 then tmsg = tmsg + ' 1-d '
 if vdim(0) ne 0 then tmsg = tmsg + ' scalar'
 print,tmsg
; print,'Type .con or retall to continue'
; stop
;
 retall
 end  ; pcheck

;************************************************************************
;+
;*NAME:
;
;	IUEGETOPT	(FORMERLY GETOPT)
;
;*CLASS:
;
;*CATEGORY:
;
;*PURPOSE:
;
;	Procedure to convert a user's response string into a valid
;	scalar or vector.  Distinct elements in the string may be
;	separated by either a comma or a space.  The output scalar
;	or vector can be specified to be  either  integer or floating
;	point.   A null string is converted to a zero.   !ERR is set
;	to the number of elements supplied.  If an input
;	is invalid, !ERR is set to -1 and the result is set to 999.
;
;*CALLING SEQUENCE:
;
;	IUEGETOPT,INPUT,OPTION	              ; Floating point output assumed
;	IUEGETOPT,INPUT,OPTION,type           ; Specify output type 
;	IUEGETOPT,INPUT,OPTION,type,numopt    ; Truncate output vector to
;					         NUMOPT elements
;
;*PARAMETERS:
;
;	INPUT	(I) (REQ)  string that was input by user in response to a prompt
;
;	OPTION	(O) (REQ)  scalar or vector containing the numeric conversion
;			   of the fields in the string INPUT.  If NUMOPT is not
;			   supplied, the number of elements in OPTION will 
;			   equal the number of distinct fields in INPUT.
;
;	TYPE	(I) (REQ)  "I" = integer, "F" = floating point
;
;	NUMOPT	(O) (OPT)  number of values expected by calling procedure
;			   If less than NUMOPT values are supplied the output
;			   vector will be padded with zeros.  
;
;*EXAMPLES:
;
;*SYSTEM VARIABLES USED:
;
;	!ERR
;
;*INTERACTIVE INPUT:
;
;*SUBROUTINES CALLED:
;
;	PARCHECK
;	IUEGETTOK
;
;*FILES USED:
;
;*SIDE EFFECTS:
;
;	!ERR reset.
;
;*RESTRICTIONS:
;
;*NOTES:
;
;	tested with IDL Version 2.1.0 (sunos sparc)	21 Jun 91
;	tested with IDL Version 2.1.0 (ultrix mispel)	N/A
;	tested with IDL Version 2.1.0 (vms vax)		21 Jun 91
;
;*PROCEDURE:
;
;*I_HELP  nn:
;
;*MODIFICATION HISTORY:
;
;	written by B. Pfarr, STX, 5/6/87
;	change value of !ERR W. Landsman   STX,  6/30/88
;	 2-21-91 PJL converted to sun/unix; renamed from GETOPT to
;		 IUEGETOPT; RDAF prolog added; added PARCHECK
;	 5-23-91 PJL changed functions IUEGETOPT and IUEGETTOK to procedures
;	 6-21-91 PJL tested on SUN and VAX; updated prolog
;
;-
;*************************************************************************
 pro iuegetopt,input,option,type,numopt
;
 if n_params(0) eq 0 then begin
    print,'IUEGETOPT,OPTION,INPUT,type,numopt'
    retall
 endif  ; n_params(0)
 parcheck,n_params(0),[2,3,4],'IUEGETOPT'
 on_ioerror,nogood
 !err = 0
;
 inp = input
;
;  look for comma
;
 comma = strpos(inp,',')
;
;  Delineator is comma or space
;
 if comma gt 0 then char = ',' else char = ' '
 if n_params(0) lt 3 then option = fltarr(10) else  $
;
;  Default type is float
;
 if strupcase(type) eq 'I' then option = intarr(10) $
    else option = fltarr(10)
;
;  Null string is 0.0
;
 if strlen(inp) eq 0 then begin
    option = 0.0
    return
 endif else begin
    i =0
    while inp ne '' do begin
       iuegettok,inp,char,token
       token = strtrim(token,2)
       if token ne '' then begin
          option(i) = float(token)
          i = i+1
       endif  ; token
    endwhile  ; inp
 endelse  ; srtlen
;
 !err = i
 if i eq 1 then option=option(0) else $               ;Scalar supplied?
    if n_params(0) lt 4 then option = option(0:i-1)  $   ;Trim output vector
       else option=option(0:numopt-1) 
    return                  ;Successful completion
;
 nogood:        ;Invalid input
    !err = -1
    option = 999.9
    return
 end  ; iuegetopt

;*********************************************************************
;+
;
;*NAME:
;
;    PARCHECK     (GENERAL IDL LIBRARY 01)  30-MAR-1987
;
;*CLASS:
;
;    Error checking
;
;*CATEGORY:
; 
;*PURPOSE:
;
;    To check that a procedure has been called with the minimum of allowed
;    number of parameters. 
; 
;*CALLING SEQUENCE:
;
;    PARCHECK,NPARM,MINPARMS,CALLINGPRO
; 
;*PARAMETERS:
;
;    NPARM       (REQ) (I) (0) (I)
;                required input scalar giving the number of parameters
;                in the procedure call (i.e. n_params(0)).
;
;    MINPARMS    (REQ) (I) (0 1) (I)
;                If scalar, the minimum number of parameters needed for the
;                procedure to execute properly.
;                If an array, it represents the allowed numbers of
;                parameters (e.g. if 3,4 or 6 parameters are allowed,
;                then set minparms([0,1,2]) = [3,4,6] ).
;
;    CALLINGPRO  (REQ) (I) (0) (S)
;                Required string giving the name of the calling procedure.
;
;*EXAMPLES:
;
;    To determine if procedure pro, which contains a
;    call to parcheck has the minimum number of parameters
;    (i.e. 4):
;
;             PARCHECK,N_PARAMS(),4,'PRO'
;
;    If the same procedure can have 4,5,7, or 8 parameters
;    then use:
;
;             PARCHECK,N_PARAMS(),[4,5,7,8],'PRO'
;
;*SYSTEM VARIABLES USED:
;
;*INTERACTIVE INPUT:
;
;*SUBROUTINES CALLED:
;
;    PCHECK
;
;*FILES USED:
;
;*SIDE EFFECTS:
;
;*RESTRICTIONS:
;
;*NOTES:
;
;    tested with IDL Version 2.1.0  (sunos sparc)     	28 Jun 91
;    tested with IDL Version 2.1.0  (ultrix mipsel)   	N/A
;    tested with IDL Version 2.1.0  (vax vms)         	28 Jun 91
;
;*PROCEDURE:
;
;    The input parameters to PARCHECK are first checked themselves
;    using PCHECK. If MINPARMS is a scalar it is compared to NPARM.
;    If NPARM < MINPARMS, then an error message is printed and the
;    procedure returns to the main level. If MINPARMS is a vector,
;    then NPARM is subtracted from each value of MINPARMS and the
;    resulting vector is checked for zeroes. If no values are zero,
;    then error messages are printed and the program returns to the
;    main level.
; 
;*MODIFICATION HISTORY :
;
;    Mar 30 1987    cag  gsfc  initial program
;    Apr    1987    rwt  gsfc  add vector input for parameters
;    Mar 15 1988    cag  gsfc  add vax rdaf-style prolog
;    Jul 12 1989    jtb  gsfc  converted to sun/unix idl
;    Nov  2 1989    rwt  gsfc  correct print format syntax
;    May 10 1991    PJL  GSFC  corrected prolog format
;    Jun 21 1991    gra  casa  cleaned up; tested SUN, DEC, VAX;
;                              updated prolog.
;    Jun 28 1991    PJL  GSFC  added npar test; tested on SUN and VAX; 
;			       updated prolog
;
;-
;***********************************************************************
 pro parcheck,nparm,minparms,callingpro
;
 npar = n_params(0)
 if npar eq 0 then begin
    print,'PARCHECK,NPARM,MINPARMS,CALLINGPRO'
    retall
 endif  ; npar
;
; check input parameters
;
 pcheck,nparm,1,100,0111
 pcheck,minparms,2,110,0111
 pcheck,callingpro,3,100,1000
 cpro = strupcase(callingpro)
;
; check if number of parameters is > minimum (if minparms is scalar)
;
 s = size(minparms)
;
 if s(0) eq 0 then begin
   if nparm lt minparms then begin
      print,format='(1x,3a,i2,a)','Procedure ',cpro,  $
        ' needs at least ',fix(minparms),' parameters to execute.'
      print,format='(1x,a,i2,a)','Only ',fix(nparm),' parameters were specified'
      print,'Action: Returning to the main program level'
      retall
   endif  ; nparm lt minparms
;
; check if nparm is an allowed number of parameters (if minparms is a vector)
;
 endif else begin
   ind = where(minparms - nparm,nonz) ; nonz will be # of non-zero values
   if s(1) eq nonz then begin
      print,'Invalid number of input parameters for procedure ',cpro
      print,'Action: Returning to the main program level'
      retall
   endif  ; s(1) eq nonz
 endelse  ; nparm scalar or vector
;
 return
 end  ; parcheck

;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
      FUNCTION ten,dd,mm,ss
;+
; NAME:
;	TEN()
; PURPOSE:
;	Converts a sexigesimal number to decimal.
; EXPLANATION:
;	Inverse of the SIXTY() function.
;
; CALLING SEQUENCES:
;	X = TEN( [ HOUR_OR_DEG, MIN, SEC ] )
;	X = TEN( HOUR_OR_DEG, MIN, SEC )
;	X = TEN( [ HOUR_OR_DEG, MIN ] )
;	X = TEN( HOUR_OR_DEG, MIN )
;	X = TEN( [ HOUR_OR_DEG ] )      <--  Trivial cases
;	X = TEN( HOUR_OR_DEG )        <--
;
; INPUTS:
;	HOUR_OR_DEG,MIN,SEC -- Scalars giving sexigesimal quantity in 
;		in order from largest to smallest.    
;
; OUTPUTS:
;	Function value returned = double real scalar, decimal equivalent of
;	input sexigesimal quantity.  A minus sign on any nonzero element
;	of the input vector causes all the elements to be taken as
;	< 0.
;
; PROCEDURE:
;	Mostly involves checking arguments and setting the sign.
;
;	The procedure TENV can be used when dealing with a vector of 
;	sexigesimal quantities.
;
; MODIFICATION HISTORY:
;	Written by R. S. Hill, STX, 21 April 87       
;	Modified to allow non-vector arguments.  RSH, STX, 19-OCT-87
;	Converted to IDL V5.0   W. Landsman   September 1997
;-
      np = N_params()

      if (np eq 1) then begin            
         vector=dd
      endif else begin
         if (np lt 1) or (np gt 3) then goto,bad_args
         vector=dblarr(3)
         vector[0]=dd
         vector[1]=mm
         if np gt 2 then vector[2]=ss
      endelse
      sz = size(vector)
      ndim = sz[0]
      if (ndim eq 0) then return,double(vector)
      facs=[1.0d0,60.0d0,3600.0d0]
      nel = sz[1]
      sign = +1.0d0
      signflag = total(vector lt dblarr(nel))
      if (signflag gt 0.0d0) then sign = -1.0d0
      vector = abs(vector)
      decim = double(vector[0])
      i = 1
      while (i le nel-1) do begin
         decim = decim + double(vector[i])/facs[i]
         i = i + 1
      endwhile
      return,decim*sign
bad_args:    
      print,'Argument(s) should be hours/degrees, minutes (optional),'
      print,'seconds (optional)   in vector or as separate arguments.'
      print,'If any one number negative, all taken as negative.'
      return,0.0d0     
      end

;@chara_config.pro
;+
; *NAME:
;	CHARA_PLAN
;
;       Three Parts: (1) chara_event.pro (this manages the GUI) 
;                    (2) chara_plan.pro (this builds the GUI)
;                    (3) chara_config.pro (this does the calculations
;                        and plotting)
;
; *PURPOSE:  A GUI interface to the chara_config routine to planning observations
;	    with the CHARA inteferometric array
;	
;
; *CATEGORY: planning tool
;	
;
; *CALLING SEQUENCE: chara_plan
;	
;
; *COMMON BLOCKS:
;  common nums  - lists of input values
;  common save_vars - saved parameter values set by user
;
;
; MODIFICATION HISTORY:
;
; 1.0 beta (28/feb/04) 
;
; Major revision, HD catalog and angular diameter lists added (jpa, 27/feb/04) 
;
; first version written by jpa, 8/dec/03
;-
pro chara_event, ev


;#
;# Common Blocks to pass arrays of stars names, baselines, POP value,
;# etc from the GUI it self chara_plan.pro (see below)
;#
  common nums,star_nums,ad_nums,baseline_nums,year_nums,month_nums,day_nums,plot_nums,$
    t1_pop_nums,t2_pop_nums,t1_pop_names,t2_pop_names,band_nums
  common save_vars,star,ad,baseline,year,month,day,plot_num,t1_pop_name,$
    t2_pop_name,t1_pop,t2_pop,band_num

;#
;#  chara_event runs the widget, keeping track of what
;#  values are selected for the lists and buttoms
;#

;#
;# ev.id identifies the event with the list or buttom was just selected
;#
;# get the name of the widget selected and put it into the variable 'value'
;#
  widget_control, ev.id, get_uvalue=value,/hourglass

;#
;# identify the type of event 
;#
  if (n_elements(value) eq 0) then value = ''
  name = strmid(tag_names(ev, /structure_name), 7, 1000)

;#
;#  Two basic actions 
;#
;#  (1) click in the plot window
;#  (2) selection of a list or buttom 
;#

  case (name) of
;#
;# If the there's a mouse click in the plot window, give the data co-
;# ordiates of the location
;#

  "DRAW": begin
        cursor,xposition,yposition,4
        value = string(format='("X: ",f7.2,'$
                + ' " Y: ",f7.2)',xposition,yposition)
	end
  else: begin

;#
;# Otherwise check which list/buttom was selected
;#

;#
;# if the quit buttom is hit, destroy the whole widget!
;#
      if (value eq 'quit') then begin
          WIDGET_CONTROL, /destroy, ev.top
          return
      endif


;#
;# Parameter buttoms
;#
;      if (value eq 'star')     then star=star_nums(ev.index)  ;# HD number has changed
      
      if (value eq 'star')     then begin
          star=long(ev.value(0)) ;# HD number has changed
;          print,ev.type
;          print,star
;          help,star
      endif

;      if (value eq 'ad')       then ad=ad_nums(ev.index)      ;#
;      Angular diameter has changed

      if (value eq 'ad')       then begin
          ad=ev.value(0) ;# Angular diameter has changed
;          print,ev.type
;          print,ad
;          help,ad
      endif

      if (value eq 'year')     then year=year_nums(ev.index)  ;# Year has changed
      if (value eq 'month')    then month=month_nums(ev.index);# Month has changed
      if (value eq 'day')      then day=day_nums(ev.index)    ;# Day has changed
      if (value eq 'plot')    then plot_num=plot_nums(ev.index);# Plot type has changed
      if (value eq 'band')     then band_num=band_nums(ev.index) ;# Waveband has changed
      if (value eq 'baseline') then baseline=baseline_nums(ev.index) ;# Baseline has changed
      if (value eq 't1_pop')   then begin          ;# Telescope 1 POP has changed
          t1_pop=t1_pop_nums(ev.index) 
          t1_pop_name=t1_pop_names(ev.index)
      endif
      if (value eq 't2_pop')     then begin        ;# Telescope 2 POP has changed            
          t2_pop=t2_pop_nums(ev.index)
          t2_pop_name=t2_pop_names(ev.index)
      endif

;#
;# Check for previously set parameters
;#
      check_star=size(star)
      check_ad=size(ad)
      check_baseline=size(baseline)
      check_year=size(year)
      check_month=size(month)
      check_day=size(day)
      check_plot_num=size(plot_num)
      check_band_num=size(band_num)
      check_t1_pop=size(t1_pop)
      check_t2_pop=size(t2_pop)
      check_t1_pop_name=size(t1_pop_name)
      check_t2_pop_name=size(t2_pop_name)
 
;#
;# For those parameters not yet set by the user, use the defaults
;#          
      if (check_star(1) eq 0) then in_star=48915 else in_star=star
      if (check_ad(1) eq 0) then in_ad=5.6 else in_ad=ad
      if (check_baseline(1) eq 0) then in_baseline=baseline_nums(0) else in_baseline=baseline
      if (check_year(1) eq 0) then in_year=year_nums(0) else in_year=year
      if (check_month(1) eq 0) then in_month=month_nums(0) else in_month=month
      if (check_day(1) eq 0) then in_day=day_nums(0) else in_day=day
      if (check_plot_num(1) eq 0) then in_plot_num=plot_nums(0) else in_plot_num=plot_num
      if (check_band_num(1) eq 0) then in_band_num=band_nums(0) else in_band_num=band_num
      if (check_t1_pop(1) eq 0) then pop1=t1_pop_nums(0) else pop1=t1_pop
      if (check_t2_pop(1) eq 0) then pop2=t2_pop_nums(0) else pop2=t2_pop
      if (check_t1_pop_name(1) eq 0) then pop_name1=t1_pop_names(0) else pop_name1=t1_pop_name
      if (check_t2_pop_name(1) eq 0) then pop_name2=t2_pop_names(0) else pop_name2=t2_pop_name

;#
;# The total POP delay is the *difference* of the delays for telescope 1 and
;# telescope 2
;#
      in_delay=(-1.d0*pop1)+pop2
      pop_config=pop_name1+' & '+pop_name2
;#
;# all inputs are set, call chara_config and generate the plot!
;#

      ;Save plot to PS file?
      if (value eq 'make_ps') then begin
          name_out='HD'+strtrim(string(in_star),2)+'.ps'
          jplotopen,name_out,/ps
          WIDGET_CONTROL, ev.top, get_uvalue=out_text
          WIDGET_CONTROL, out_text , set_value='Plotting to file: '+name_out,/append
      endif

      chara_config,in_star,in_ad,in_baseline,in_year,in_month,in_day,$
        in_plot_num,in_delay,pop_config,in_band_num,visvec,sfvec,pavec,$
        havec,basename,utrange,uttime,cart_delay,/noprint,error

    WIDGET_CONTROL, ev.top, get_uvalue=out_text
    WIDGET_CONTROL, out_text , set_value=error,/append

    ;Close plotting device
    if (value eq 'make_ps') then begin
        jplotclose
        WIDGET_CONTROL, ev.top, get_uvalue=out_text
        WIDGET_CONTROL, out_text , set_value='...done.',/append

      endif

  end

endcase

if (name eq 'DRAW') then begin 
    WIDGET_CONTROL, ev.top, get_uvalue=out_text
    WIDGET_CONTROL, out_text , set_value=string(value),/append
endif

end

pro chara_plan

  common nums,star_nums,ad_nums,baseline_nums,year_nums,month_nums,day_nums,plot_nums,$
    t1_pop_nums,t2_pop_nums,t1_pop_names,t2_pop_names,band_nums
  common save_vars,star,ad,baseline,year,month,day,plot_num,t1_pop_name,$
    t2_pop_name,t1_pop,t2_pop,band_num

; next line checks to make sure the current graphics
; driver can run idl widgets.  If not, it ends.
  if(!D.FLAGS and 65536) eq 0 then return

  base = WIDGET_BASE(title='CHARA Observation Planning Tool')
  b1 = WIDGET_BASE(base, /frame, /column)
  bt = WIDGET_BASE(b1, /frame, /row)


  t1 = WIDGET_TEXT(bt, xsize=75, ysize=5, /SCROLL, $
      value=[ 'Welcome to the CHARA Planning Tool',$
              'Beta 1.7 8/apr/04 jasona@noao.edu ', $
	'', $
	'    (1) Select a Waveband. Enter HD Number, and Angular Size on the right. ',$
        '         Defaults: K-band and Sirius',$
        '         Only HD numbers in the HIPPARCOS catalog are valid.',$
	'    (2) Select a Baseline and POP configuration at the bottom left.  ',$
        '         Defaults: S1-S2 and S1 POP1 and S2 POP2',$
	'         Check that Baseline and POP selections are consistent',$ 
	'    (3) Select a Plot Type and Date at the bottom right.',$
        '         Defaults: (U,V) Plane, 2003, January, 1',$
  	'    (4) Click in plot window to get X and Y values.',$
  	'    (5) Click on "Make of PS File" to generate a postscript file',$
  	'        of the current plot.',$
'',$
'Explanation of Plots:',$
'(1) (U,V) Plane: Circles are J1 function zeros',$
'       crosses mark hour angles with no OPLE/POP coverage',$
'(2) Spatial Freq. vs. Altitude: Labels are hour angles',$
'(3) Vis. vs. Hour Angle: ',$
'       vertical lines indicate daylight',$
'       crosses mark hour angles with no OPLE/POP coverage',$
'(4) Vis. vs. Spatial Frequency:',$
'       crosses mark hour angles with no OPLE/POP coverage',$
'(5) Altitude vs. Azimuth:',$
'       crosses mark hour angles with no OPLE/POP coverage',$
'(6) Position Angle vs Hour Angle:',$
'       vertical lines indicate daylight',$
'       crosses mark hour angles with no OPLE/POP coverage',$
'(7) Altitude and Delay vs. UT time:',$
'       -43 to +43 meters is the delay available from carts.',$
'       Delay available when white line is between these values.'])


  t2 = WIDGET_TEXT(bt, xsize=35, ysize=5, /SCROLL, $
      value=[ 'Output Window'])

  WIDGET_CONTROL, base, set_uvalue=t2





  b2 = WIDGET_BASE(b1, /frame, /row)
  draw = widget_draw(b2, xsize=600, ysize=500, /button_events)
  b3 = widget_base(b2, /column, /frame, space=10)


;  hd_num = CW_FIELD(b3, TITLE='HD Number',uvalue='star',value='48915',$
;                    /COLUMN,/LONG,/RETURN_EVENTS)

;  ud_size =CW_FIELD(b3, TITLE='UDD (mas)',uvalue='ad',value='5.6',$
;                    /COLUMN,/FLOAT,/RETURN_EVENTS)

;#
;# Create list of wave bands
;#
bands=['K-band','H-band']
band_nums=[0,1]
drop_band = widget_droplist(b3,VALUE=bands,UVALUE='band',TITLE='Band')


;#
;# create a list of stars (all the HD stars in the HIPPARCOS catalog)
;#

  restore,'hipp_ra_dec_hd_spec.idl' ;ra (deg), dec (deg), HD number, spectral type
  k=where(hipp_dec gt -30.d0)
  i=sort(hipp_hd(k))
  star_nums=hipp_hd(k(i))
  stars=strtrim(string(star_nums),2)
;  label_star = widget_label(b3,VALUE='Target HD Number',/align_center)
;  list_star = widget_list(b3,VALUE=stars,UVALUE='star',ysize=10)

;
; Enter you own HD number!
;
  list_star = cw_field(b3,TITLE='Target HD Number',UVALUE='star',ysize=1,/return_events,/column,/long)


;#
;# create a list of angular diameters 0.1 to 30.0 mas
;#
  ad_nums=[findgen(91)/100+0.1,findgen(290)/10+1.1]
  ads=strmid(strtrim(string(ad_nums),2),0,4)
;
; OLD VERSION (LIST) 
;
;  label_ad = widget_label(b3,VALUE='Angular Diameter',/align_center)
;  label_ad = widget_label(b3,VALUE='(mas)',/align_center)
;  list_ad = widget_list(b3,VALUE=ads,UVALUE='ad',ysize=10)

;
; Enter your own Angular Diameter
;
  list_star = cw_field(b3,TITLE='Angular Diameter (mas)',UVALUE='ad',ysize=1,/return_events,/column,/float)


  make_ps_button = widget_button(b3, value='Make PS file', uvalue='make_ps')

  quit_button = widget_button(b3, value='QUIT', uvalue='quit')


;#
;# Build widget bases for baselines, POPs, date and Plot type
;#

b3=widget_base(b1, /row)

;#
;# create a list of CHARA Baselines
;#

baselines=[' 34.09 m, 350.1 deg: S1-S2',' 65.88 m, 236.5 deg: E1-E2',$
'110.88 m, 97.5  deg: W1-W2',$
'152.22 m, 63.3  deg: W2-E2','178.39 m, 340.2 deg: S2-W2','212.04 m, 341.8 deg: S1-W2',$
'217.76 m, 241.2 deg: E1-W2','248.11 m, 17.7  deg: S2-E2','249.38 m, 317.0 deg: S2-W1',$
'251.31 m, 77.6  deg: W1-E2','278.50 m, 320.9 deg: S1-W1','278.76 m, 14.5  deg: S1-E2',$
'302.30 m, 25.5  deg: S2-E1','313.50 m, 253.2 deg: E1-W1','330.65 m, 22.1  deg: S1-E1']

baseline_nums=indgen(15)+1 ;associated number values (1-15)
b31 = widget_base(b3, /column)
label_baseline = widget_label(b31,VALUE='Baseline',/align_center)
list_baseline = WIDGET_LIST(b31, VALUE=baselines, UVALUE='baseline',YSIZE=3)

;#
;# create a list of POP positions
;#

t1_pop_names=['S1 POP1   0.000 m','S1 POP2  36.576 m','S1 POP3  73.152 m',$
'S1 POP4 109.728 m',$
'S1 POP5 146.304 m','S2 POP1 -36.576 m','S2 POP2   0.000 m','S2 POP3  36.576 m',$
'S2 POP4  73.152 m','S2 POP5 109.728 m','E1 POP1   0.000 m','E1 POP2  36.576 m',$
'E1 POP3  73.152 m','E1 POP4 109.728 m','E1 POP5 146.304 m','E2 POP1 -73.152 m',$
'E2 POP2 -36.576 m','E2 POP3   0.000 m','E2 POP4  36.576 m','E2 POP5  73.152 m',$
'W1 POP1 -73.152 m','W1 POP2 -36.576 m','W1 POP3   0.000 m','W1 POP4  36.576 m',$
'W1 POP5  73.152 m','W2 POP1 -146.304 m','W2 POP2 -109.728 m','W2 POP3  -73.152 m',$
'W2 POP4  -36.576 m','W2 POP5    0.000 m']

t2_pop_names=t1_pop_names

; t1_pop_short_names=['S1 POP1','S1 POP2','S1 POP3','S1 POP4',$
; 'S1 POP5 ','S2 POP1','S2 POP2','S2 POP3',$
; 'S2 POP4 ','S2 POP5','E1 POP1','E1 POP2',$
; 'E1 POP3 ','E1 POP4','E1 POP5','E2 POP1',$
; 'E2 POP2 ','E2 POP3','E2 POP4','E2 POP5',$
; 'W1 POP1 ','W1 POP2','W1 POP3','W1 POP4',$
; 'W1 POP5 ','W2 POP1','W2 POP2','W2 POP3',$
; 'W2 POP4 ','W2 POP5']
 
;t2_pop_short_names=t1_pop_short_names

t1_pop_nums=[ 0.000  ,  36.576  ,  73.152 , 109.728, 146.304, -36.576,   0.000,  36.576,$
              73.152, 109.728,   0.000,  36.576,  73.152, 109.728, 146.304, -73.152,$
              -36.576,   0.000,  36.576,  73.152, -73.152, -36.576,   0.000,  36.576,$
              73.152,-146.304,-109.728, -73.152, -36.576,   0.000]

t2_pop_nums=t1_pop_nums

b32 = widget_base(b3, /column)
label_t1_pops = widget_label(b32,VALUE='Telescope 1 POP',/align_center)
list_t1_pops = WIDGET_LIST(b32, VALUE=t1_pop_names, UVALUE='t1_pop',YSIZE=3)
b33 = widget_base(b3, /column)
label_t2_pops = widget_label(b33,VALUE='Telescope 2 POP',/align_center)
list_t2_pops = WIDGET_LIST(b33, VALUE=t2_pop_names, UVALUE='t2_pop',YSIZE=3)

;#
;# Create widget base for Year, Month and Day
;#

;b4 = widget_base(b1, /row, /frame, space=10)
b5 = widget_base(b3, /column, /frame, space=4)

;#
;# Create list of plot types
;#
plots=['(U,V) Plane','Spatial Freq.vs. Altitude','Vis. vs. Hour Angle',$
       'Vis. vs. Spatial Freq.','Altitude vs. Azimuth','Pos. Angle vs. Hour Angle','Altitude/Delay vs. UT']
plot_nums=[0,1,2,3,6,7,8]
drop_plot = widget_droplist(b5,VALUE=plots,UVALUE='plot',TITLE='Plot')

b4 = widget_base(b5, /row, /frame, space=4)

;#create list of years
years=['2003','2004','2005','2006','2007']
year_nums=[2003,2004,2005,2006,2007]
drop_year = widget_droplist(b4,VALUE=years,UVALUE='year');,TITLE='Year')

;#create list of months
months=['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']
month_nums=[1,2,3,4,5,6,7,8,9,10,11,12]
drop_month = widget_droplist(b4,VALUE=months,UVALUE='month');,TITLE='Month')

;#create list of days
days = [' 1',' 2',' 3',' 4',' 5',' 6',' 7',' 8',' 9','10','11','12','13','14',$
            '15','16','17','18','19','20','21','22','23','24','25','26','27','28','29','30','31']
day_nums=indgen(31)+1
drop_day = widget_droplist(b4,VALUE=days,UVALUE='day');,TITLE='Day')



;#
;# envoke the GUI!
;#

widget_control, base, /realize

WIDGET_CONTROL, get_value=window, draw

;#
;# Set Default Values  (for Sirius)
;#
star=48915
ad=5.6
baseline=baseline_nums(0)
year=year_nums(0)
month=month_nums(0)
day=day_nums(0)
plot_num=plot_nums(0)
band_num=band_nums(0)
delay=t1_pop_nums(0)+t2_pop_nums(6)
t1_pop_name=t1_pop_names(0)
t2_pop_name=t2_pop_names(6)
pop_config=t1_pop_name+' & '+t2_pop_name

;#
;# call chara_config
;#
chara_config,star,ad,baseline,year,month,day,plot_num,delay,pop_config,$
band_num,/noprint

;#
;# start the GUI manager! (see chara_event.pro above) 
;#

XMANAGER,'CHARA', base,group_leader=group

;# that's it, the GUI is built.
end

;@orbit2 ;orbit for Spica
;@sunpos ;for twilight calculations
;+
; *NAME: chara_config -- this routine is called by the IDL GUI
;                       "chara_plan.pro".  This routine does the
;                       actual calculations 
;
; A CHARA Array observation planning tool
;
;                by
;
;         Jason Aufdenberg             
;  National Optical Astronomy Observatory 
;         jasona@noao.edu
;
; For a range of hour angles from -6h to +6h compute visibilities for 
; a star, given the HD number, angular size, baseline, POP
; configuration, and date.  The user has a choice of plots.
;
; *PURPOSE:  show U,V tracks, spatial freuqucies, and predicted
; visibilties for a given source 
;
;
;
; *CATEGORY:  Observation planning tool
;
;
;
; *CALLING SEQUENCE: 
;                   
;     chara_config,HRNUM,UD,NUM,YEAR,MONTH,DAY,PTYPE,POP_DELAY,POP_CONFIG,
;                 WAVEBAND,VISVEC,SFVEC,AVEC,HAVEC,BASENAME,UTRANGE,UTIME,CART_DELAY
;                 NOPRINT=NOPRINT
;
; *INPUTS: HDNUM  = Henry Draper  number of target [integer]
;
;         UD    = Uniform Disk Angular Diameter (in mas)  [real]
;
;         NUM    = number corresponding to baseline [integer]
;                            1 --> S1-S2
;                            2 --> E1-E2
;                            3 --> W1-W2
;                            4 --> W2-E2
;                            5 --> S2-W2
;                            6 --> S1-W2
;                            7 --> E1-W2
;                            8 --> S2-E2
;                            9 --> S2-W1
;                           10 --> W1-E2
;                           11 --> S1-W1
;                           12 --> S1-E2
;                           13 --> S2-E1
;                           14 --> E1-W1
;                           15 --> S1-E1
;
;        YEAR  = year of observation (eg. 2004) [integer or real]
;
;        MONTH = month observation [1-12]  [integer or real]
;
;        DAY   = day of month observation (e.g. 1-28,29,30,31)
;                [integer or real] 
;
;        PTYPE = type of plot requested [integer]
;            0 ---> U (cycles/arcssec) vs. V (cycles/arcsecond)  
;            1 ---> Spatial frequency (cycles/arcsec) vs. Altitude (degrees)
;            2 ---> Squared Visibility  vs. Hour  Angle (hours)
;            3 ---> Squared Visibility  vs. Spatial Frequency (cycles/arcsecond)
;            4 ---> *plot nothing, just return output vectors*
;            5 ---> *plot nothing, just return output vectors*
;            6 ---> Altitude (degrees) vs. Azimuth (degrees)
;            7 ---> Position Angle (degrees) vs. Hour Angle (hours)
;            8 ---> Altitude (degrees) and Delay (meters) vs. UT Time (hours)
;            9 ---> *plot nothing, but compute observable UT time
;                    range for this baseline, POP configuration
;
;        POP_DELAY  = delay offset (in meters) provided by a given
;                    P.O.P. configuration (e.g.,  S1/POP1
;                    0.000m + S2/POP1 -36.576 m = -36.576 m)  [real]
;
;            Here's the table to figure the total POP delay by hand.
;                The GUI "chara_plan.pro" does this for you.
;
;                   POP1     POP2      POP3      POP4      POP5  
;           S1:   0.000 m   36.576 m  73.152 m 109.728 m  146.304 m
;           S2: -36.576 m    0.000 m  36.576 m  73.152 m  109.728 m
;           E1:   0.000 m   36.576 m  73.152 m 109.728 m  146.304 m
;           E2: -73.152 m  -36.576 m   0.000 m  36.576 m   73.152 m
;           W1: -73.152 m  -36.576 m   0.000 m  36.576 m   73.152 m
;           W2:-146.304 m -109.728 m -73.152 m -36.576 m    0.000 m
;
;
;        POP_CONFIG = a string to label the value for POP_DELAY (e.g.,
;                    'S1/1 & S2/1')
;                    The GUI "chara_plan.pro" adds this string for you.
;
;        WAVEBAND   = waveband for observation [integer]
;                     0 --> 'K' band  (2.2 microns)
;                     1 --> 'H' band  (1.6 microns)
;
;
; *OPTIONAL INPUTS: NONE
;                     
;
; *KEYWORD PARAMETERS: /NOPRINT  =  don't print table of U,V, PA, HA,
;                                 Alt, V, V^2, etc.
;
; *OUTPUTS:  SFVEC      = vector of spatial frequencies, sqrt(U^2 + V^2),
;                      (in cycles/arcsecond)              
;                               
;           PAVEC      = vector of position angles (PA = arctangent (U/V)) (in radians)
;                        
;           HAVEC      = vector of hour angles (-6 hours to +6 hours)                      
;
;           BASENAME   = baseline name corresponding to value   of NUM 
;
;           UTRANGE    = vector containing lower and upper time limits
;                        observability in UT
;
;           CART_DELAY =  -1/2 *( geometric delay- 
;                               - air_difference 
;                               + total P.O.P delay)  for the given baseline
;    
; *OPTIONAL OUTPUTS: NONE
;
;
;
; *EXAMPLE:  for Sirius = alpha CMa = HD 48915 plot U,V track for the
;           S1-W2 baseline on 2004 February 1st, no P.O.P delay
;
;    chara_config,48915,5.6,11,2004,2,1,0,0.0,'',0,/noprint
;
; *MODIFICATION HISTORY:
;
; 28/feb/04: beta 1.0
;
; 27/feb/04: change fixed set of HR numbers to every HD number in the HIPPARCOS
;            catalog with Dec > -30!  User must now input angular
;            diameter
;
; 13/feb/04: massive improvement in the comments! Cleaned up the code,
;            plot labels are more readable
;
;  2/jan/04: Added waveband as input, choose K and H-band.  Added band option
;            into the GUI chara_plan.
;
; 17/dec/03: At CHARA itself! Twilight code tested last week, seems to work.
;            updated to include computation of geometric delay.
;            With input from chara_plan.pro, POP delay compenstation configuration
;            is considered and hour angle coverage is computed for
;            each baseline/POP configuration.  Coverage is quite
;            limited, especially with W2.  Careful planning required.
;
; 1/dec/03:  added beta version for  calculation of UT time and
;            astronomical twilight
;
; Bug Fix: 14/nov/03:  Baseline labels S2-E2 and S1-E2 were switched.
;
; Update: 14/nov/03:   W1 on-line, first light on W2 soon.
;                         
; Reality Check update: 28/mar/03.  From Steve: S3 was not funded and
; was not built.  S2 is currenty off-line.  W1 is coming in months,
; W2 coming in the next year.
;
; First version (jpa, 2/sep/02)
;
;-


pro chara_config,hdnum,ud,num,year,month,day,ptype,pop_delay,pop_config,$
                 waveband,visvec,sfvec,pavec,havec,basename,utrange,uttime,$
                 cart_delay,noprint=noprint,error

;# 
;# if user calls 'chara_config' with no or wrong number of parameters, print help
;#

npar=n_params(0)
if (npar eq 0 or npar lt 10) then begin
print,'chara_config,HRNUM,UD,NUM,YEAR,MONTH,DAY,PTYPE,'
print,'             POP_DELAY,POP_CONFIG,WAVEBAND,NOPRINT=NOPRINT'
print,'             visvec,sfvec,pavec,havec,basename,utrange,utime,cart_delay'
print,'' 
print,' *EXAMPLE:  for Sirius = alpha CMa = HD 48915 plot U,V track for the'
print,'            S1-W2 baseline on 2004 February 1st, no P.O.P delay'
print,''
print,"            chara_config,48915,5.6,11,2004,2,1,0,0.0,'',0,/noprint"
print,'' 
print,"  type: doc_library,'chara_config' for full help "
retall
endif

;#
;# check user input star number
;#
;#  step #1, not an array, integer, real or double and > 0

    check=size(hdnum)
    if (check(0) ne 0 $         ; an array
                                ;not integer, real or double
        or (check(1) ne 2 and check(1) ne 4 and check(1) ne 3 and check(1) ne 5)$ 
        or (hdnum lt 0)) then begin 
        print,'chara_config: bad input HDNUM! ',hdnum
        print,'HDNUM  = Henry Draper  number of target [integer]'
        retall
    endif

;#  step #2, is this star in the HIPPARCOS main catalog?
    restore,'hipp_ra_dec_hd_spec.idl' ;ra (deg), dec (deg), HD number, spectral type
    k=where(hipp_hd eq hdnum)
    if (k(0) eq -1) then begin
;bummer! 
       print,"chara_config: can't find HD",hdnum," in HIPPARCOS catalog"
       error="Can't find HD "+strtrim(string(hdnum),2)
        return ;this allows error to be sent to GUI
;        retall
    endif else begin
;
;star found! grab coordinates from catalog
;
        ra=hipp_ra(k(0))/15.0d0           ;RA:  degrees ---> hours 
        dec=hipp_dec(k(0))*!pi/180.d0     ;DEC: degrees ---> radians
        spectype=hipp_spec(k(0))
    endelse

;#
;# check user uniform disk size
;#
    check=size(ud)
    if (check(0) ne 0 $         ;an array
        or (check(1) ne 2 and check(1) ne 4 and check(1) ne 5)$ ;not integer, real or double
        or (ud lt 0)) then begin 
        print,'chara_config: bad input UD! ',ud
        print,'UD    = Uniform Disk Angular Diameter (> 0 in mas)  [real]'
        error="Bad angular diameter: "+strtrim(string(ud),2)
        return
;        retall
    endif


;#
;#  Check user baseline number
;#
;#
    check=size(num)
    if (check(0) ne 0 $         ; an array
        or (check(1) ne 2)$     ; not integer
        or (num lt 0 or num gt 15)) then begin  
        print,'chara_config: bad input baseline NUM! ',num
        print,'NUM    = number corresponding to baseline (0 < NUM < 15) [integer]'
        print,'    1 --> S1-S2'
        print,'    2 --> E1-E2'
        print,'    3 --> W1-W2'
        print,'    4 --> W2-E2'
        print,'    5 --> S2-W2'
        print,'    6 --> S1-W2'
        print,'    7 --> E1-W2'
        print,'    8 --> S2-E2'
        print,'    9 --> S2-W1'
        print,'   10 --> W1-E2'
        print,'   11 --> S1-W1'
        print,'   12 --> S1-E2'
        print,'   13 --> S2-E1'
        print,'   14 --> E1-W1'
        print,'   15 --> S1-E1'
        retall
    endif
;#
;#  Check user input of year, month, day
;#
;#
    check=size(year)
    if (check(0) ne 0 $         ;an array
        or (check(1) ne 2 and check(1) ne 4 and check(1) ne 5)$ ;not integer, real or double
        or (year lt 1583)) then begin ;no interferometers before 1583!
        print,'chara_config: bad input YEAR! ',year
        print,'YEAR  = year of observation (>1582, eg. 2004) [integer or real]'
        retall
    endif

    check=size(month)
    if (check(0) ne 0 $         ;an array
        or (check(1) ne 2 and check(1) ne 4 and check(1) ne 5)$ ;not integer, real or double
        or (month lt 1 or month gt 12)) then begin  
        print,'chara_config: bad input MONTH ',month 
        print,'MONTH = month observation [1-12]  [integer or real]'
        retall
    endif

    check=size(day)
    if (check(0) ne 0 $         ;an array
        or (check(1) ne 2 and check(1) ne 4 and check(1) ne 5)$ ;not integer, real or double
        or (day lt 1 or day gt 31)) then begin  
        print,'chara_config: bad input DAY ',day 
        print,'DAY   = day of month observation (0 < DAY < 32) [integer or real]'
        retall
    endif

;#
;#  Check user plottype
;#
;#
    check=size(ptype)
    if (check(0) ne 0 $         ;an array
        or (check(1) ne 2)$     ; not integer
        or (ptype lt 0 or ptype gt 9)) then begin  
        print,'chara_config: bad input PTYPE ',ptype 
        print,' PTYPE = type of plot requested [integer]'
        print,'     0 ---> U (cycles/arcssec) vs. V (cycles/arcsecond)  '
        print,'     1 ---> Spatial frequency (cycles/arcsec) vs. Altitude (degrees)'
        print,'     2 ---> Squared Visibility  vs. Hour  Angle (hours)'
        print,'     3 ---> Squared Visibility  vs. Spatial Frequency (cycles/arcsecond)'
        print,'     4 ---> *plot nothing, just return output vectors*'
        print,'     5 ---> *plot nothing, just return output vectors*'
        print,'     6 ---> Altitude (degrees) vs. Azimuth (degrees)'
        print,'     7 ---> Position Angle (degrees) vs. Hour Angle (hours)'
        print,'     8 ---> Altitude (degrees) and Delay (meters) vs. UT Time (hours)'
        print,'     9 ---> *plot nothing, but compute observable UT time'
        retall
    endif


;#
;# Differential Airpaths (relative to telescope S0) from Theo's "telescopes.chara" astrometric solution
;# 
airpath=[0.d0,4532654.762d0,15309717.902d0,26721359.853d0,$  
         29087218.284d0,-8654293.636d0]/1.d6  ;s0,s1,e1,e2,w1,w2 (in meters) 

;#
;# The full length of delay provided by the delay carts
;#
ople_max=96. ; in meters 

;#
;# CHARA LATITUDE and LONGITUDE
;#
lat=ten(34.,13.,0.)*(!pi/180.d0) ; lat. in deg --> radians  (Almanac,;for Hale Obs.)
long=ten(-118.,03.,36.)*(!pi/180.d0) ;long in deg --> radians

;#
;# From the user input NUM, return baseline length (in meters), bearing (in
;# degrees), and  air path difference (in meters)
;#
;# The S3 telescope baselines are included here, but if selected the program
;# will exit.  Maybe there'll be an S3 actual one day.
;#
;# Baselines and Bearings from Table 4 of CHARA Technical Report No. 48
;# The 3D Layout of the CHARA Array" by T.A. ten Brunmmelaar
;# 
;# S1 S2 E1 E2 W1 W2
;# XYZ OFFSETS from "telescopes.chara"
;#Date: Fri, 19 Dec 2003 22:18:53 -0800


;#XOFFSET	0.0  ;S1  East
;#YOFFSET 0.0  ;S1        North
;#ZOFFSET 0.0  ;S1        Up=+ 
;#XOFFSET   -5748393.059  ;S2
;#YOFFSET   33581340.790  ;S2
;#ZOFFSET     643880.188  ;S2
;#XOFFSET	  125341890.588 ;E1
;#YOFFSET   305935660.046 ;E1
;#ZOFFSET    -5904942.288 ;E1
;#XOFFSET    70400864.470 ;E2
;#YOFFSET   269716660.932 ;E2
;#ZOFFSET    -2787652.960 ;E2
;#XOFFSET -175068583.489 ;W1
;#YOFFSET  216322063.446 ;W1
;#ZOFFSET  -10827769.936 ;W1
;#XOFFSET	 -69089002.341 ;W2
;#YOFFSET	 199350442.500 ;W2
;#ZOFFSET     470336.880 ;W2

;# OLD;
;# OLD;XOFFSET=[0.0d0, -5748393.059d0, 125341890.588d0, 70400864.470d0,$
;# OLD; -175068583.489d0,-69089002.341d0]/1.d6 
;# OLD;
;# OLD;YOFFSET=[0.0d0, 33581340.790d0, 305935660.046d0, 269716660.932d0,$
;# OLD;         216322063.446, 199350442.500d0]/1.d6
;# OLD;
;# OLD;ZOFFSET=[0.0d0, 643880.188d0, -5904942.288d0, -2787652.960d0,$ 
;# OLD;         -10827769.936d0,  470336.880d0]/1.d6 

;#
;#Date: Wed, 03 Mar 2004 11:26:46 -0800
;#
;#S1
;#LONG    -118 3 25.31272
;#LAT       34 13 27.78130
;#XOFFSET 0.0
;#YOFFSET 0.0
;#ZOFFSET 0.0
;# 
;#
;#S2
;#XOFFSET   -5748393.059
;#YOFFSET   33581340.790
;#ZOFFSET     643880.188
;# 
;#
;#E1
;#XOFFSET   125349003.099
;#YOFFSET   305937513.274
;#ZOFFSET    -5898978.192
;# 
;#
;#E2
;#XOFFSET    70400864.470
;#YOFFSET   269716660.932
;#ZOFFSET    -2787652.960
;# 
;#
;#W1
;#XOFFSET -175068583.489
;#YOFFSET  216322063.446
;#ZOFFSET  -10827769.936
;# 
;#
;#W2
;#XOFFSET -69088236.186
;#YOFFSET 199329322.146
;#ZOFFSET    451418.063


XOFFSET=[0.0d0,   -5748393.059d0, 125349003.099d0, 70400864.470d0,$
         -175068583.489d0, -69088236.186d0]/1.d6

YOFFSET=[0.0d0,   33581340.790d0,   305937513.274d0, 269716660.932d0,$
         216322063.446d0, 199329322.146d0]/1.d6

ZOFFSET=[0.0d0,    643880.188d0,   -5898978.192d0, -2787652.960d0,$
         -10827769.936d0, 451418.063d0]/1.d6

basevec=dblarr(3)
case 1 of

;"New" Telescope names

    (num eq 1 ): begin 
        basename="S1-S2" 
        baseline= 34.09 
        bearing=350.1
        basevec(0)=xoffset(0)-xoffset(1) ;east
        basevec(1)=yoffset(0)-yoffset(1) ;north
        basevec(2)=zoffset(0)-zoffset(1) ;up
        air_diff=airpath(0)-airpath(1)
    end
    (num eq 2 ): begin 
        basename="E1-E2" 
        baseline= 65.88  
        bearing=236.5  
        basevec(0)=xoffset(2)-xoffset(3) ;east
        basevec(1)=yoffset(2)-yoffset(3) ;north
        basevec(2)=zoffset(2)-zoffset(3) ;up
        air_diff=airpath(2)-airpath(3)
    end
    (num eq 3 ): begin 
        basename="W1-W2" 
        baseline=110.88  
        bearing=97.5  
        basevec(0)=xoffset(4)-xoffset(5) ;east
        basevec(1)=yoffset(4)-yoffset(5) ;north
        basevec(2)=zoffset(4)-zoffset(5) ;up
        air_diff=airpath(4)-airpath(5)
    end
    (num eq 4 ): begin 
        baseline=152.22  
        bearing=63.3  
        basename="W2-E2"  
        basevec(0)=xoffset(5)-xoffset(3) ;east
        basevec(1)=yoffset(5)-yoffset(3) ;north
        basevec(2)=zoffset(5)-zoffset(3) ;up
        air_diff=airpath(5)-airpath(3)
    end
    (num eq 5): begin 
        basename="S2-W2"
        baseline=178.39
        bearing=340.2 
        basevec(0)=xoffset(1)-xoffset(5) ;east
        basevec(1)=yoffset(1)-yoffset(5) ;north
        basevec(2)=zoffset(1)-zoffset(5) ;up
        air_diff=airpath(1)-airpath(5)
    end
    (num eq 6): begin 
        basename="S1-W2"  
        baseline=212.04  
        bearing=341.8   
        basevec(0)=xoffset(0)-xoffset(5) ;east
        basevec(1)=yoffset(0)-yoffset(5) ;north
        basevec(2)=zoffset(0)-zoffset(5) ;up
        air_diff=airpath(0)-airpath(5)
    end
    (num eq 7): begin 
        basename="E1-W2" 
        baseline=217.76  
        bearing=241.2 
        basevec(0)=xoffset(2)-xoffset(5) ;east
        basevec(1)=yoffset(2)-yoffset(5) ;north
        basevec(2)=zoffset(2)-zoffset(5) ;up
        air_diff=airpath(2)-airpath(5)
    end
    (num eq 8): begin 
        basename="S2-E2"  
        baseline=248.11  
        bearing=17.7  
        basevec(0)=xoffset(1)-xoffset(3) ;east
        basevec(1)=yoffset(1)-yoffset(3) ;north
        basevec(2)=zoffset(1)-zoffset(3) ;up
        air_diff=airpath(1)-airpath(3)
    end
    (num eq 9): begin 
        basename="S2-W1"  
        baseline=249.38  
        bearing=317.0 
        basevec(0)=xoffset(1)-xoffset(4) ;east
        basevec(1)=yoffset(1)-yoffset(4) ;north
        basevec(2)=zoffset(1)-zoffset(4) ;up
        air_diff=airpath(1)-airpath(4)
    end
    (num eq 10): begin 
        basename="W1-E2"  
        baseline=251.31  
        bearing=77.6   
        basevec(0)=xoffset(4)-xoffset(3) ;east
        basevec(1)=yoffset(4)-yoffset(3) ;north
        basevec(2)=zoffset(4)-zoffset(3) ;up
        air_diff=airpath(4)-airpath(3)
    end
    (num eq 11): begin 
        basename="S1-W1" 
        baseline=278.50   
        bearing=320.9  
        basevec(0)=xoffset(0)-xoffset(4) ;east
        basevec(1)=yoffset(0)-yoffset(4) ;north
        basevec(2)=zoffset(0)-zoffset(4) ;up
        air_diff=airpath(0)-airpath(4)
    end
    (num eq 12): begin 
        basename="S1-E2" 
        baseline=278.76  
        bearing=14.5   
        basevec(0)=xoffset(0)-xoffset(3) ;east
        basevec(1)=yoffset(0)-yoffset(3) ;north
        basevec(2)=zoffset(0)-zoffset(3) ;up
        air_diff=airpath(0)-airpath(3)
    end
    (num eq 13): begin 
        basename="S2-E1" 
        baseline=302.30  
        bearing=25.5  
        basevec(0)=xoffset(1)-xoffset(2) ;east
        basevec(1)=yoffset(1)-yoffset(2) ;north
        basevec(2)=zoffset(1)-zoffset(2) ;up
        air_diff=airpath(1)-airpath(2)
    end
    (num eq 14): begin 
        basename="E1-W1" 
        baseline=313.50  
        bearing=253.2 
        basevec(0)=xoffset(2)-xoffset(4) ;east
        basevec(1)=yoffset(2)-yoffset(4) ;north
        basevec(2)=zoffset(2)-zoffset(4) ;up
        air_diff=airpath(2)-airpath(4)
    end
    (num eq 15): begin 
        basename="S1-E1" 
        baseline=330.65  
        bearing=22.1    
        basevec(0)=xoffset(0)-xoffset(2) ;east
        basevec(1)=yoffset(0)-yoffset(2) ;north
        basevec(2)=zoffset(0)-zoffset(2) ;up
        air_diff=airpath(0)-airpath(2)
    end
endcase


;#OLD;#
;#OLD;# From the baseline and bearing compute the E and N components of the
;#OLD;# baseline vector for later computation of  (U,V) spatical frequencies 

;#
;# Debug 3-D (X,Y,Z) component baselines
;#

;DEBUG ;
;DEBUG ;basetest= sqrt(basevec(0)^2 +basevec(1)^2 +basevec(2)^2 )
;DEBUG ;print,'baseline, basetest: ',baseline,basetest
;DEBUG ;
;DEBUG ;sin_bearing=(basevec(0)/basetest)
;DEBUG ;cos_bearing=(basevec(1)/basetest)
;DEBUG ;
;DEBUG ;if(cos_bearing ge 0 and sin_bearing ge 0) then     beartest=asin(sin_bearing)*180./!pi ;1st quad
;DEBUG ;if(cos_bearing le 0 and sin_bearing ge 0) then     beartest=180.0-asin(sin_bearing)*180./!pi ;2nd quad
;DEBUG ;if(cos_bearing le 0 and sin_bearing le 0) then     beartest=180.0-asin(sin_bearing)*180./!pi ;3rd quad
;DEBUG ;if(cos_bearing ge 0 and sin_bearing le 0) then     beartest=360+asin(sin_bearing)*180./!pi ;4th quad
;DEBUG ;
;DEBUG ;beartest=beartest+180.d0
;DEBUG ;while(beartest gt 360.d0) do beartest=beartest-360.d0
;DEBUG ;
;DEBUG print,'bearing, beartest:    ',bearing,beartest
;DEBUG ;
;DEBUG;   
;DEBUG;        baselines involving W2 are different from CHARA
;DEUBG         Technical Report (which predates telescopes.dat by several years)
;DEBUG;
;DEBUG;        ----------------------------------------------------
;DEBUG;              BASELINE (m)             BEARING (degrees)
;DEBUG;        -----------------------    -------------------------
;DEBUG;                      telescopes.                 telescopes.
;DEBUG;        CTR No. 48      chara     CTR No. 48       chara
;DEBUG;S1-S2   34.0900       34.075872    350.100       350.28811
;DEBUG;E1-E2   65.8800       65.879054    236.500       236.50854
;DEBUG;W1-W2!  110.880       107.92291  ! 97.5000       100.88950
;DEBUG;W2-E2!  152.220       156.26721    63.3000       63.206478
;DEBUG;S2-W2   178.390       177.45833    340.200       339.08808
;DEBUG;S1-W2   212.040       210.98367    341.800       340.88524
;DEBUG;E1-W2!  217.760       221.82070    241.200       241.22571
;DEBUG;S2-E2   248.110       248.13378    17.7000       17.871781
;DEBUG;S2-W1   249.380       249.38945    317.000       317.23922
;DEBUG;W1-E2   251.310       251.33817    77.6000       77.594079
;DEBUG;S1-W1   278.500       278.49863    320.900       321.05194
;DEBUG;S1-E2   278.760       278.76716    14.5000       14.628081
;DEBUG;S2-E1   302.300       302.33165    25.5000       25.696091
;DEBUG;E1-W1   313.500       313.53036    253.200       253.36629
;DEBUG;S1-E1   330.650       330.66915    22.1000       22.275177


;# X-Y only component baselines
;# OLDbasevec=dblarr(2)
;# OLDbasevec(0)=sin(!pi*(bearing+180.)/180.)*baseline ;east component of baseline vector 
;# OLDbasevec(1)=cos(!pi*(bearing+180.)/180.)*baseline ;north component of baseline vector 


;#
;# from the WAVEBAND user input, select K or H band
;#

case 1 of
    (waveband eq 0): begin 
        lam=2.2*1.e-6  &  bandname='K'   ;wavelength in meters and label
    end
    (waveband eq 1): begin 
        lam=1.6*1.e-6   &   bandname='H'  ;wavelength in meters and label
    end

    (waveband gt 1): begin 
    print,"{waveband} must be '0' for K-band or '1' for H-band, try again." & retall

end

endcase

    
;OLD;;#
;OLD;;#
;OLD;;# Get basic star data using user input HRNUM
;OLD;;# 
;OLD;;#
;OLD;
;OLD;
;OLD;
;OLD; get_star_data,hrnum,name2,ra,dec,bmag,vmag,spectype,rv,erv,plx,eplx,kmag,ud,eud,hd,name3,bad
;dec=dec*!pi/180.; declination (degrees --> radians)

;#
;# debug declination
;#
;# decsix=sixty(dec*180.d0/!pi)
;# print,'DEC. = '+string(decsix,format='(i3," d ",i2," m ",f5.2," s")')
;#


;#######################################################################################################
;# Compute local mean sidereal time for a given date
;########################################################################################################
;# We need this to convert hour angles to UT times for a given date
;#
;#  See "Greenwich Mean Sidereal Time GMST" (on Page B6 from the Astronomical Almanac)
;#  under TIME-SCALES: "Relationships between universal and sidereal time"
;#

;# debug ipout values
;# year=2003
;# month=12
;# day=18
;# hour=0

;#vector of month labels
month_vec=['Jan','Feb','Mar','Apr','May','Jun','Jul','Aug','Sep','Oct','Nov','Dec']

;#
;# use ASTROLIB idl routine 'juldate' to compute the modified julian
;# date (MJD)at UT=0h for the users inputs YEAR, MONTH, DAY
;#
;#   from the 'juldate' documentation header:  
;#   
;# >     A date of 25-DEC-1981 06:25 UT may be expressed as either
;# >
;# >    IDL> juldate,[81,12,25,6,25],jd       
;# >    IDL> juldate,[1981,12,25.2673611],jd 
;# >
;# >    In either case, one should obtain a reduced Julian Date of 
;# >    JD = 44963.7673611
;#
;#  So MJD = JD - 200000.0   
;#    

juldate,[year,month,day,0,0,0],mjd

;#
;# print,'MJD = ',mjd
;# 

;#
;# Here we follow the Astronomical Almanac Formulae
;#

tu=(mjd-51545.d0)/36525.d0  ;# Julian centuries of 36525 days since ;J2000.0

;# Greenwich Mean Sidereal Time at Oh UT (in seconds)
gmst= 24110.54841d0 + (8640184.812866d0*tu) + (0.093104d0*tu*tu) + (6.2d-6*tu*tu*tu)

;# Greenwich Mean Sidereal Time in hours
;# 
;# checked against USNO on-line software
;# 
gmst_h=(gmst/3600.d0) 

;#
;# reduced hours to between 0h and 24h
;#

while(gmst_h gt 24) do     gmst_h=gmst_h-24.d0         ;
while(gmst_h lt 0) do     gmst_h=gmst_h+24.d0        

;#
;# Local Mean Sidereal Time in hours at 0h UT (longitude correction)
;#
lmst_h=(gmst/3600.d0)+((long*180./!pi)/15.d0)

;#
;# reduced hours to between 0h and 24h
;#
while(lmst_h gt 24) do lmst_h=lmst_h-24.d0 ;
while(lmst_h lt 0) do lmst_h=lmst_h+24.d0 ;

;#
;# compute the hour angle of the target star at Oh UT
;#
;# We use this to compute the UT times which correspond the vector 
;# of hour angles constructed below
;#

ha_at_ut_zero=lmst_h-ra  ;HA of star at 0h UT on MJD

;#####################################################################################
; end time calcucations
;#####################################################################################

;# mm = size of vectors hours angles (from -6 h  to +6 h, see below) 
;#      are divided into 48 parts (one point every 15 minutes)
mm=48 
;#Special case: Polaris
if (hdnum eq 8890) then begin
    mm=96
endif


;mm=144 ;one point every 5 minutes 
;#
;#
;# Initialize variables (vectors) to compute at each hour angle
;#
;#
sfvec=dblarr(mm+1)              ;vector to save spatial frequenices sf = sqrt(u^2 + v^2) (cycles/arcsec)
havec=dblarr(mm+1)              ;vector to save hour angles (in hours)
altvec=dblarr(mm+1)             ;vector to save altitudes   (in degrees)
azvec=dblarr(mm+1)              ;vector to save azimuths    (in degrees)
u_vec=dblarr(mm+1)              ;vector of save U component of spatial frequency (cycles/arcsec)
v_vec=dblarr(mm+1)              ;vector of save V component of spatial frequency (cycles/arcsec)
pavec=dblarr(mm+1)              ;vector of save position angles (degrees). note: PA=arctangent(U/V)
visvec=dblarr(mm+1)             ;vector of squared visibilities (V^2)
visvec_ld=dblarr(mm+1)          ;vector of limb-darkened squared visibilities
gdvec=dblarr(mm+1)              ;vector of geometric delays (in meters)
vel1vec=dblarr(mm+1)            ;vector of primary component velocity
vel2vec=dblarr(mm+1)            ;vector of secondary component velocity



;# 
;#  TOP of Hour Angle loop.  
;# 
for m=0,mm do begin 

;#
;#   mm = 48  (see above)
;#
;#  compute hour angle (in radians)
;#
;#  from m=0,48, ha goes from -6 h to +6 h,one  point every 15 minutes.
;#
    ha=(0.25*m-6)*(15.d0*!pi/180.d0) ; hour angle in hours --> radians
;    ha=(0.083333*m-6)*(15.d0*!pi/180.d0) ; hour angle in hours --> radians

    if (hdnum eq 8890) then begin ;#special case for Polaris
        ha=(0.25*m-12)*(15.d0*!pi/180.d0) ; hour angle in hours --> radians
    endif
;#
;# Compute Altitude at this hour angle (see Page B61, Astrononmical Almanac)
;#
    sin_altitude=sin(dec)*sin(lat)+cos(dec)*cos(ha)*cos(lat)
    altitude=asin(sin_altitude)*180./!pi ;altitude in degress

;#
;# Compute Azimuth at this hour angle (see Page B61, Astrononmical Almanac)
;#
    sin_azimuth=-1.d0*cos(dec)*sin(ha)/cos(asin(sin_altitude))
    cos_azimuth= sin(dec)*cos(lat)-cos(dec)*cos(ha)*sin(lat)
;#
;#  pick the right quadrant for the azimuth!
;#  also convert to degrees from radians
;#
    if(cos_azimuth ge 0 and sin_azimuth ge 0) then     azimuth=asin(sin_azimuth)*180./!pi         ;1st quad
    if(cos_azimuth le 0 and sin_azimuth ge 0) then     azimuth=180.0-asin(sin_azimuth)*180./!pi   ;2nd quad
    if(cos_azimuth le 0 and sin_azimuth le 0) then     azimuth=180.0-asin(sin_azimuth)*180./!pi   ;3rd quad
    if(cos_azimuth ge 0 and sin_azimuth le 0) then     azimuth=360+asin(sin_azimuth)*180./!pi     ;4th quad

;#debug#
;#debug#    print,'azimuth= ', cos_azimuth, sin_azimuth, azimuth
;#debug#;   print,'H.A. = ',ha*180./(!pi*15.),'Alt. = ',altitude,$
;#debug#;         format='(a,f5.2," h",2x,a,f5.2," d")'


;# 
;# Compute U,V coordinates for this Hour Angle
;# 
;# Mel Dyck's talk  "Interferometry with Two Telescopes" in
;# Principles of Long Baseline Stellar Interferometry (equation 12.1, page 189)
;#

;#
;# both u and v are in cycles/arcsecond
;#

;# OLD X-Y components ;
;# OLD X-Y components ;
;# OLD X-Y components ;;# U
;# OLD X-Y components ;    u=(basevec(0)*cos(ha) - basevec(1)*sin(lat)*sin(ha))/(206265.*lam) 
;# OLD X-Y components ;;# V
;# OLD X-Y components ;    v=(basevec(0)*sin(dec)*sin(ha) + $
;# OLD X-Y components ;       basevec(1)*(sin(lat)*sin(dec)*cos(ha) +$
;# OLD X-Y components ;                       cos(lat)*cos(dec)))/(206265.*lam)

;#
;# NEW X-Y-Z components
;#

;# U
    u=(basevec(0)*cos(ha) - basevec(1)*sin(lat)*sin(ha) +$
       basevec(2)*cos(lat)*sin(ha) )/(206265.*lam) 
;# V
    v=(basevec(0)*sin(dec)*sin(ha) + $
       basevec(1)*(sin(lat)*sin(dec)*cos(ha) + cos(lat)*cos(dec)) +$
       basevec(2)*(cos(lat)*sin(dec)*cos(ha) - sin(lat)*cos(dec)))/(206265.*lam)

;#  gdelay =  delay (in meters) for a baseline vector with  north,
;# east  (and now north earth and "up") components.  A star in the west will
;# have a positive delay, while a star in the east will be negative delay. 
;#
;#OLD XY only delay ;    gdelay= - basevec(0)*cos(dec)*sin(ha) - $
;#OLD XY only delay ;      basevec(1)*(sin(lat)*cos(dec)*cos(ha) - cos(lat)*sin(dec))  
    gdelay= - basevec(0)*cos(dec)*sin(ha) - $
     basevec(1)*(sin(lat)*cos(dec)*cos(ha) - cos(lat)*sin(dec))  +$
     basevec(2)*(cos(lat)*cos(dec)*cos(ha) + sin(lat)*sin(dec))  


;#
;# spatial frequency for a symmetric source
;#
    sf=sqrt(u^2 + v^2)

;#
;#  kwa = argument of the bessel function.  
;#  2*pi*(Baseline/lambda) [in radians] * uniform disk radius [in radius]
;#
    kwa=sf*2.0*!pi*206265*ud*0.5e-3/206265

;#
;#  V^2: squared visibility for a uniform disk.... as simple as it gets.
;#
    v2=(2.*beselj(kwa,1)/(kwa))^2.0

;#
;# projected baseline (in meters)
;#
    pbase=sf*206265.*lam
;#


;#
;#
;# Position angle (East of North)
;#
;#
    pa=atan(u/v)*180./!pi 
;#
;# when plotting the position angle it's nice to keep the angles between -60 to 120
;# since the position angle of the projected baseline is same when
;# rotated 180 degrees 
;#
;#
    if (pa le -60) then pa=pa+180


;###################################
;# SPECIAL CASE:  BINARITY
;###################################

    if (hdnum eq 116658) then begin

;# Let's make Spica a real binary!



;# U.D. angular diameter (in mas) for primary (Spica A)
        ud=0.87 ;*1.06
        ud_1=ud  
;# U.D. angular diameter (in mas) for secondary (Spica B)
        ud_2=0.4 

;# Brightness ratio (at K?)
        r=(1./6.4)            ;brightness ratio

;# arguments for Bessel function
        kwa_1=sf*2.0*!pi*206265*ud_1*0.5e-3/206265
        kwa_2=sf*2.0*!pi*206265*ud_2*0.5e-3/206265


;# uniform disk visibilties for primary        
        v2_1=(2.*beselj(kwa_1,1)/(kwa_1))^2.0
;# uniform disk visibilties for secondary
        v2_2=(2.*beselj(kwa_2,1)/(kwa_2))^2.0

;# find UT at for each hour angle

        uttime=((ha*180./(!pi*15.))-ha_at_ut_zero)/1.002737 
        time=mjd+((uttime)/24.d0)

;#  call 'orbit2' program to find the apparent binary
;# separation on the sky (in arcseconds) and the position angle (in
;# radians) on the line connecting the two stars at this
;# hour angle step
;#
        orbit2,time,rho,theta
;#
;# convert apparent separation and position angle into delta right
;# ascension and delta declinations (both in arcseconds)
;#
        delta_RA=rho*sin(theta)
        delta_DEC=rho*cos(theta)

;# From Andy Boden's notes from the Michelson Summer School 1999
;# in the book "Principles of Long Baseline Stellar Interferometry"
;# Chapter 2, Equation (2.32).
;#

        v2_binary=(v2_1 + ((r^2)*v2_2) + (2.d0*r*sqrt(v2_1)*sqrt(v2_2)*$
                                          cos(2.d0*!pi*((u*delta_RA) +$
                                                        (v*delta_DEC)))))/((1.d0+r)^2)
;# assign visbility with binary visiblity
        v2=v2_binary

    endif ;That's if for Spica!

;#
;# How will alpha Oph companion affect things?
;# The ZPD is to too large to see fringes from both stars.
;#

;#OFF#
;#OFF#    if (hdnum eq 159561) then begin
;#OFF#
;#OFF#;# Let's make Alpha Oph a real binary!
;#OFF#
;#OFF#
;#OFF#;# U.D. angular diameter (in mas) for primary (Spica A)
;#OFF#        ud=1.53
;#OFF#        ud_1=ud  
;#OFF#;# U.D. angular diameter (in mas) for secondary (Spica B)
;#OFF#        ud_2=0.5
;#OFF#
;#OFF#;# Brightness ratio  (based on delta K = 3.5 mags)
;#OFF#        r=(1./25011.)            ;brightness ratio
;#OFF#
;#OFF#;# arguments for Bessel function
;#OFF#        kwa_1=sf*2.0*!pi*206265*ud_1*0.5e-3/206265
;#OFF#        kwa_2=sf*2.0*!pi*206265*ud_2*0.5e-3/206265
;#OFF#
;#OFF#
;#OFF#;# uniform disk visibilties for primary        
;#OFF#        v2_1=(2.*beselj(kwa_1,1)/(kwa_1))^2.0
;#OFF#;# uniform disk visibilties for secondary
;#OFF#        v2_2=(2.*beselj(kwa_2,1)/(kwa_2))^2.0
;#OFF#
;#OFF#;#
;#OFF#;# companion position from Boccaletti et al (2001)
;#OFF#;#
;#OFF#        rho=0.77                ;arcseconds
;#OFF#        theta=116*!pi/180.d0    ;degrees --> radians
;#OFF#
;#OFF#;#
;#OFF#;# convert apparent separation and position angle into delta right
;#OFF#;# ascension and delta declinations (both in arcseconds)
;#OFF#;#
;#OFF#        delta_RA=rho*sin(theta)
;#OFF#        delta_DEC=rho*cos(theta)
;#OFF#
;#OFF#;# From Andy Boden's notes from the Michelson Summer School 1999
;#OFF#;# in the book "Principles of Long Baseline Stellar Interferometry"
;#OFF#;# Chapter 2, Equation (2.32).
;#OFF#;#
;#OFF#
;#OFF#        v2_binary=(v2_1 + ((r^2)*v2_2) + (2.d0*r*sqrt(v2_1)*sqrt(v2_2)*$
;#OFF#                                          cos(2.d0*!pi*((u*delta_RA) +$
;#OFF#                                                        (v*delta_DEC)))))/((1.d0+r)^2)
;#OFF#;# assign visbility with binary visiblity
;#OFF#        v2=v2_binary
;#OFF#
;#OFF#    endif ;That's if for Alpha Oph
;#OFF#

;#debug#
;#debug# to_pb=206265.*lam
;#debug# if (not keyword_set(noprint)) then begin
;#debug#     print,lam/1.e-9,u*to_pb,v*to_pb,sf,pbase,pa,$
;#debug#           ha*180./(15*!pi),altitude,azimuth,sqrt(v2),v2,format='(11(f8.2,2x))'
;#debug# endif

;#
;# Print Header AND dump ASCII for most important variables
;#

    if (not keyword_set(noprint)) then begin

        if (mm eq 0) then begin
            print,"lambda","U","V","sf","baseline","pos angle","H.A","Alt","Az","V","V^2",format='(11(a8,2x))'
            print,"[nm]","[1/arcsec]","[1/arcsec]","[1/arcsec]","[m]","[deg]","[h]","[deg]","[deg]",$
              "V","V^2",format='(11(a8,2x))'
        endif
        print,lam/1.e-9,u,v,sf,pbase,pa,ha*180./(15*!pi),altitude,azimuth,sqrt(v2),v2,format='(11(f8.2,2x))'
    endif

;#
;# Save values at this hour angle step into vectors
;#


    havec(m)=ha*180./(!pi*15.)  ;hour angle [in hours]
    visvec(m)=sqrt(v2)          ;visibility for uniform disks
    u_vec(m)=u                  ;U [in cycles/arcsec]
    v_vec(m)=v                  ;V [in cycles/arcsec]
    sfvec(m)=sf                 ;spatial frequency [in cycles/arcsec]
    pavec(m)=pa                 ;position angle [in degrees]
    altvec(m)=altitude          ;altitude [in degrees]
    azvec(m)=azimuth            ;azimuth [in degrees]
    gdvec(m)=gdelay             ;geometric delay [in meters]

;# BOTTOM of hour angle loop
endfor

;#
;# Find where the star has altitudes > 30.
;#

kk=where(altvec gt 30.)

;######################################################################################
;# OPLE Cart Delay Computation
;######################################################################################
;# We've found the hour angles where the altitude is
;# greater than 0.  Now we'd like to see at which hour angles are
;# available given constraints of the baseline (OPLE delay including
;# POP configuration).
;#
;# This bit of the code was tested in December 2003 against the software
;# running on the CHARA observer's computer which plots Delay vs. UT Time
;# and Altitude of star vs. UT Time.
;#

;# the needed delay to be provided by the carts
    cart_delay=-0.5*(gdvec(kk)-air_diff+pop_delay)

        kbad=where(cart_delay gt ople_max/2.0 or $
                   cart_delay lt -ople_max/2.d0)

;# W2 cart now working 1/apr/04 
;#OFF#
;#OFF#    if (num ne 7 and num ne 6 and num ne 5 and num ne 4 and num ne 3) then begin
;#OFF#;# in this convention, the cart delay runs from -43 to +43 meters with
;#OFF#;# with the total OPLE cart delay of 96 meters
;#OFF#;#
;#OFF#;# bad hour angles are those where the cart delay exceeds this range
;#OFF#;# (ople_max and the aif_diff variables are assigned near the top of the program)
;#OFF#;#
;#OFF#        kbad=where(cart_delay gt ople_max/2.0 or $
;#OFF#                   cart_delay lt -ople_max/2.d0)
;#OFF#    endif else begin
;#OFF#;# W2 cart is currently (17/Dec/03) immobile, so any baselines with
;#OFF#;# this station have only 1/2 the full OPLE delay coverage.
;#OFF#        kbad=where(cart_delay gt 0.or $
;#OFF#                   cart_delay lt -ople_max/2.d0)
;#OFF#    endelse
;######################################################################################
;######################################################################################


;#
;# Compute the spatial frequencies of the circles for this baseline, lambda, and uniform disk size  
;#

;# converation constant:  milli-arcseconds --> radians
mas_to_rad=1e-3/206265.d0 

;#locations of the zeros of the J_1 function (Abramowitz and Stegun
;#Table 9.5 page 4.09)
j1_zeros=[3.83170597d0,7.01558667d0,10.17346814d0,13.32369194d0,16.4706030d0,$
          19.615859d0,22.76008d0,25.90357d0,29.04683,29.04683d0, 32.18968d0]

;#spatical frequencies corresponding to these zeros
sf_zeros=(j1_zeros/(ud*0.5*mas_to_rad))/(2.d0*!pi*206265.d0)


;#######################################################################################################
;#
;#  Now plot what the user wants based on the value the user input PTYPE
;#


;# Label containing the Declination, Uniform Disk Size, Wavelength and
;# Baseline, and Date 
    date='  '+strtrim(string(year),2)+' ' +month_vec(month-1)+ '  ' +strtrim(string(day),2)

    bigname='HD '+strtrim(string(hdnum),2)+'('+spectype+', !4d!3='+$
      strmid(strtrim(string(dec*180./!pi),2),0,3)+$
      '!Uo!N, AD='+$
      strmid(strtrim(string(ud),2),0,3)+' mas)'+$
      bandname+'-band, '+string(basename,format='(3(a,2x))')+date

    notsobigname='HD '+strtrim(string(hdnum),2)+'('+spectype+', !4d!3='+$
      strmid(strtrim(string(dec*180./!pi),2),0,3)+$
      '!Uo!N, AD='+$
      strmid(strtrim(string(ud),2),0,3)+' mas)'+$
      bandname+'-band, '+string(basename,format='(3(a,2x))')



;# construct date label


;####################################################################################
;#
;# PLOT U vs. V
;#
;#

if (ptype eq 0) then begin


    ;cleanplot
    !x.charsize=1.2
    !y.charsize=1.2


;#
;# plot U vs V ! 
;#

;#ORIG    plot,u_vec,v_vec,psym=4,xtitle='U [cycles/arcsec]',$
;#ORIG      ytitle='V [cycles/arcsec]',xrange=[-700,700],yrange=[-700,700],$
;#ORIG      /iso,charsize=1.25,symsize=1.5,title=notsobigname

;#better plotting limits
    help=[max(abs(u_vec)),max(abs(v_vec))]
    limit=max(help)

    plot,u_vec(kk),v_vec(kk),psym=4,xtitle='U [cycles/arcsec]',$
      ytitle='V [cycles/arcsec]',xrange=[-1.2*limit,1.2*limit],yrange=[-1.2*limit,1.2*limit],$
      /iso,charsize=1.25,symsize=1.5,title=notsobigname

us=10.*findgen(201)-1000 ;vector of U or V values -1000 to +1000 (cycles/arsec)
sfs=fltarr(201,201) ;array of spatial frequencies for each point in the U, V plane

;# compute the spatial frequency at each U,V point
for ii=0,200 do begin
    for jj=0,200 do begin
        sfs(ii,jj)=sqrt(us(ii)^2 + us(jj)^2)
    endfor
endfor

;# overplot zeros contours on U,V plan 
    contour,sfs,us,us,/overplot,levels=sf_zeros

    if (kbad(0) ne -1) then begin
        oplot,u_vec(kk(kbad)),v_vec(kk(kbad)),psym=7,symsiz=2.0
    endif
    legend,['POP Coverage','No POP Coverage'],$
      psym=[4,7];,position=[0.45,0.8]

endif

;###################################################################################################
;#
;#
;# PLOT Spatial Frequency vs. Altitude
;#
;#

if (ptype eq 1) then begin

;# big labels
    !x.charsize=1.3
    !y.charsize=1.3

;# PLOT Spatial Frequency vs. Altitude

    plot,altvec(kk),sfvec(kk),psym=-4,xtitle='Altitude [degrees]',ytitle='Spatial Frequency [1/"]',$
      charsize=1.5,title=notsobigname,/ysty,yrange=[0,max(sfvec(kk))*1.1],symsize=1.5

;# overplot those points for which delay compentation is not available
;# with 'X's. 

;    if (kbad(0) ne -1) then begin
;        oplot,altvec(kk(kbad)),sfvec(kk(kbad)),psym=7,symsiz=2.0
;        legend,[pop_config],box=0
;        legend,['POP Coverage','No POP Coverage'],psym=[4,7],position=[0.5,0.8]
;        legend,[pop_config,'POP Coverage','No POP Coverage'],$
;          psym=[0,4,7],line=[-1,0,0],position=[0.45,0.8]
;    endif

;# label the points with hour angles

;all of them
;    xyouts,altvec(kk),(sfvec(kk)-sfvec(kk(0))*.05),string(havec(kk),format='(f4.1,"h")'),alignment=0.5

;on the hour
    kha=where(abs(havec(kk)-fix(havec(kk))) le 1e-3)
    xyouts,altvec(kk(kha)),(sfvec(kk(kha))-sfvec(kk(kha(0)))*.05),string(havec(kk(kha)),format='(f4.1,"h")'),alignment=0.5

;#
;#  Draws lines of constant spatial frequency the at zeros...
;#
    oplot,[-90,90],[sf_zeros(0),sf_zeros(0)],line=3
    oplot,[-90,90],[sf_zeros(1),sf_zeros(1)],line=3
    oplot,[-90,90],[sf_zeros(2),sf_zeros(2)],line=3
    oplot,[-90,90],[sf_zeros(3),sf_zeros(3)],line=3

;#
;# Label these lines
;#
    if (sf_zeros(0) le !y.crange(1)) then begin
        xyouts,!x.crange(0)+1,sf_zeros(0),'1st zero',charsize=2
    endif
    if (sf_zeros(1) le !y.crange(1)) then begin
        xyouts,!x.crange(0)+1,sf_zeros(1),'2nd zero',charsize=2
    endif
    if (sf_zeros(2) le !y.crange(1)) then begin
        xyouts,!x.crange(0)+1,sf_zeros(2),'3rd zero',charsize=2
    endif

endif

;#################################################################################################
;
; VISIBILITY V.S. HOUR ANGLE PLOT

if (ptype eq 2) then begin

; character size etc
;    cleanplot
    !x.charsize=1.3
    !y.charsize=1.3
    oldmargin=!y.margin
    !y.margin=[5,5]


;#ORIG    plot,havec(kk),visvec(kk),psym=-4,xtitle='Hour Angle [hours]',ytitle='Visibility',charsize=1.5,$
;#ORIG      title=' ',/ysty,yrange=[0,1.45],xstyle=8,symsize=1.5

    plot,havec(kk),visvec(kk),psym=-4,xtitle='Hour Angle [hours]',ytitle='Visibility',charsize=1.5,$
      title=' ',/ysty,yrange=[0,max(visvec(kk)*1.5)],symsize=1.5,xstyle=8

    !y.margin=oldmargin

;# overplot those points for which delay compentation is not available
;# with 'X's. 

    if (kbad(0) ne -1) then begin
        oplot,havec(kk(kbad)),visvec(kk(kbad)),psym=7,symsiz=2.0
    endif
    legend,['POP Coverage','No POP Coverage'],$
      psym=[4,7],position=[0.45,0.8]


;# top x-axis plot Universal time 
;#
;# This bit of logic ties to keep the UT values on the top x-axis
;# between 0 and 24h.
;#
    min_axis_ut=(!x.crange[0]-ha_at_ut_zero)/1.002737
        
;# if the UT values are more negative than -6 h, then add 24 hours 
        if (min_axis_ut lt -6) then begin
            axis,xaxis=1,xrange=((!x.crange-ha_at_ut_zero)/1.002737)+24,xstyle=1,xtitle='UT [h]'
        endif else begin
            axis,xaxis=1,xrange=((!x.crange-ha_at_ut_zero)/1.002737),xstyle=1,xtitle='UT [h]'
        endelse


;#ORIG    xyouts,min(havec(kk))-1,1.7,bigname,charsize=1.3
;#ORIG    xyouts,min(havec(kk))-1,1.625,pop_config,charsize=1.3

        xyouts,min(havec(kk)),1.7*max(visvec(kk)),bigname,charsize=1.3
        xyouts,min(havec(kk)),1.625*max(visvec(kk)),pop_config,charsize=1.3


;# plot legend
;    legend,[name2 + '  ' +date + ' UT',$
;            '!4d!3='+strmid(strtrim(string(dec*180./!pi),2),0,3)+'!Uo!N',$
;            'AD='+strmid(strtrim(string(ud),2),0,3)+' mas',$
;            bandname+'-band',$
;            string(basename,format='(3(a,2x))'),pop_config],box=0,charsize=1.8


endif

;#############################################################################################
;# VISIBILITY V.S. SPATIAL FREQUENCY
;#

if (ptype eq 3) then begin
    
;#
;# character size etc
;#
    ;cleanplot
    !x.charsize=1.3
    !y.charsize=1.3
    plot,sfvec(kk),visvec(kk),psym=-4,xtitle='Spatial Frequency [1/"]',ytitle='Visibility',charsize=1.5,$
         title=notsobigname,/ysty,yrange=[0,max(visvec(kk)*1.5)],symsize=1.5

;# overplot those points for which delay compentation is not available
;# with 'X's. 

;    if (kbad(0) ne -1) then begin
;        oplot,sfvec(kk(kbad)),visvec(kk(kbad)),psym=7,symsiz=2.0
;    endif
;        legend,[pop_config],box=0
;       legend,['POP Coverage','No POP Coverage'],psym=[4,7],position=[0.2,0.2]
;        legend,[pop_config,'POP Coverage','No POP Coverage'],$
;          psym=[0,4,7],line=[-1,0,0],position=[0.45,0.8]
; 



    if (kbad(0) ne -1) then begin
        oplot,sfvec(kk(kbad)),visvec(kk(kbad)),psym=7,symsiz=2.0
    endif
    legend,['POP Coverage','No POP Coverage'],$
      psym=[4,7]
endif


if (ptype eq 4 or ptype eq 5) then begin
;# do nothing!  except for returning sfvec,pavec,havec,basename... 
endif


;#####################################################################################################
;# ALTITUDE VS. AZMIMUTH

if (ptype eq 6) then begin

    plot,azvec(kk),altvec(kk),psym=4,xtitle='Azimuth [degrees]',ytitle='Altitude [degrees]',charsize=1.25,$
      title=notsobigname
;      title=' ',/ysty,xrange=[0,360.],yrange=[0,90],/xsty


    date=strtrim(string(year),2)+' ' +month_vec(month-1)+ '  ' +strtrim(string(day),2)

;# overplot those points for which delay compentation is not available
;# with 'X's. 

    if (kbad(0) ne -1) then begin
        oplot,azvec(kk(kbad)),altvec(kk(kbad)),psym=7,symsiz=2.0
    endif
    legend,[pop_config,'POP Coverage','No POP Coverage'],$
      psym=[0,4,7],line=[-1,0,0],position=[0.3,0.25]



endif

;#####################################################################################################
;# POSITION ANGLE V.S. HOUR  ANGLE
;#

if (ptype eq 7) then begin

; character size etc    
    ;cleanplot
    !x.charsize=1.3
    !y.charsize=1.3
    oldmargin=!y.margin
    !y.margin=[5,5]

    plot,havec(kk),pavec(kk),psym=-4,xtitle='Hour Angle [hours]',$
      ytitle='Position Angle [degrees]',$
      charsize=1.5,/ysty,yrange=[-100,200],xstyle=8,symsize=1.5,$
      title=''

    !y.margin=oldmargin

;# overplot those points for which delay compentation is not available
;# with 'X's. 

    if (kbad(0) ne -1) then begin
        oplot,havec(kk(kbad)),pavec(kk(kbad)),psym=7,symsiz=2.0
    endif
    legend,['POP Coverage','No POP Coverage'],$
      psym=[4,7],position=[0.45,0.8]


;# top x-axis plot Universal time 
;#
;# This bit of logic ties to keep the UT values on the top x-axis
;# between 0 and 24h.
;#
    min_axis_ut=(!x.crange[0]-ha_at_ut_zero)/1.002737
        
;# if the UT values are more negative than -6 h, then add 24 hours 
        if (min_axis_ut lt -6) then begin
            axis,xaxis=1,xrange=((!x.crange-ha_at_ut_zero)/1.002737)+24,xstyle=1,xtitle='UT [h]'
        endif else begin
            axis,xaxis=1,xrange=((!x.crange-ha_at_ut_zero)/1.002737),xstyle=1,xtitle='UT [h]'
        endelse
;#
;# plot title
;#
    xyouts,min(havec(kk))-1,250,bigname,charsize=1.3
    xyouts,min(havec(kk))-1,230,pop_config,charsize=1.3

endif

;###################################################################################################
;#
;#  Plot Needed Cart Delay and Altitude vs. UT Time
;#
;####################################################################################################

if (ptype eq 8) then begin
; character size etc
    ;cleanplot
    oldmargin=!x.margin
    !x.margin=[10,10]
    min_axis_ut=(min(havec(kk))-ha_at_ut_zero)/1.002737

;# if the UT values are more negative than -6 h, then add 24 hours 
        if (min_axis_ut lt -6) then begin
           uttime=((havec-ha_at_ut_zero)/1.002737)+24
        endif else begin
            uttime=((havec-ha_at_ut_zero)/1.002737)
        endelse
;
    plot,uttime(kk),altvec(kk),psym=-4,xtitle='UT [h]',$
      ytitle='Altitude [degrees]',yrange=[-46,90],symsize=1.5,ystyle=9,title=bigname,charsize=1.25

    axis,yaxis=1,ystyle=1,ytitle='Delay [meters]',charsize=1.25

    !x.margin=oldmargin

    oplot,uttime(kk),cart_delay
    oplot,[!x.crange(0),!x.crange(1)],[46,46]


;# overplot those points for which delay compentation is not available
;# with 'X's. 

    if (kbad(0) ne -1) then begin
        oplot,uttime(kk(kbad)),altvec(kk(kbad)),psym=7,symsiz=2.0
    endif
        legend,[pop_config],box=0
        legend,['POP Coverage','No POP Coverage'],psym=[4,7],position=[0.2,0.2]
    legend,['Altitude','Delay'],line=[0,0],psym=[4,0],position=[0.2,0.8]

endif



if (ptype eq 9) then begin

;find min and max UT times star is oservable for given
; baseline/POP setup

;    uttime=((havec-ha_at_ut_zero)/1.002737)+24.
    uttime=((havec-ha_at_ut_zero)/1.002737)

    cart_delay=-0.5*(gdvec-air_diff+pop_delay)

;#W2 cart is OK now
;#OFF#    if (num ne 7) then begin
        kgood=where(cart_delay lt ople_max/2.0 and $
                   cart_delay gt -ople_max/2.d0 and altvec ge 30.d0)
;#OFF#
;#OFF#    endif else begin
;#OFF#        kgood=where(cart_delay lt 0. and $
;#OFF#                   cart_delay gt -ople_max/2.d0 and altvec ge 30.d0)
;#OFF#    endelse
    
    if (kgood(0) ne -1) then begin
        ut_good=uttime(kgood)
        utrange=[min(ut_good),max(ut_good)]
    endif else begin
        utrange=[-1,-1]
    endelse

endif



if (ptype eq 2 or ptype eq 7 ) then begin
;=========================================================================================
;# BEGIN Twilight Calculations
;========================================================================================
;#
;# !fancy!  Mark Astronomical Twilight: where zenith distance of sun < 108 degrees  
;#

    sunpos,mjd,lat,long,tw_dusk,tw_dawn,lmst_dusk,lmst_dawn
;#
;# compute HAs of dusk and dawn  
;#
    ha_dusk=lmst_dusk-ra
    ha_dawn=lmst_dawn-ra

    if (ha_dusk lt min(havec(kk))) then ha_dusk=ha_dusk+24
    if (ha_dusk gt max(havec(kk))) then ha_dusk=ha_dusk-24

    if (ha_dawn lt min(havec(kk))) then ha_dawn=ha_dawn+24
    if (ha_dawn gt max(havec(kk))) then ha_dawn=ha_dawn-24

    if (ha_dawn lt -12) then ha_dawn=ha_dawn+24
    if (ha_dawn gt 12) then ha_dawn=ha_dawn-24

    if (ha_dusk lt -12) then ha_dusk=ha_dusk+24
    if (ha_dusk gt 12) then ha_dusk=ha_dusk-24


    if (ha_dusk gt !x.crange[0] and ha_dusk lt !x.crange[1]) then begin
        oplot,[ha_dusk,ha_dusk],[!y.crange[0],!y.crange[1]],line=1 
        xyouts,ha_dusk+.25,0.75*(!y.crange[0]+!y.crange[1]),'Twilight Ends',orientation=270,$
          charsize=1.8

        kday=where(havec(kk) lt ha_dusk)
    
        if (kday(0) ne -1 and n_elements(kday) gt 1)  then begin 
            for q=0,n_elements(kday)-1 do begin
                oplot,[havec(kk(kday(q))),havec(kk(kday(q)))],[!y.crange[0],!y.crange[1]]
            endfor
        endif
    endif

    if (ha_dawn gt !x.crange[0] and ha_dawn lt !x.crange[1]) then begin
        oplot,[ha_dawn,ha_dawn],[!y.crange[0],!y.crange[1]],line=1 
        xyouts,ha_dawn-0.35,0.75*(!y.crange[0]+!y.crange[1]),'Twilight Begins',orientation=270,$
          charsize=1.8

        kday=where(havec(kk) gt ha_dawn)
    
        if (kday(0) ne -1)  then begin ;if not totally dark during this hour angle range
            for q=0,n_elements(kday)-1 do begin
                oplot,[havec(kk(kday(q))),havec(kk(kday(q)))],[!y.crange[0],!y.crange[1]]
            endfor
        endif
    endif


;
; all day light...
;
    if (ha_dawn gt !x.crange[1] and ha_dawn-24 gt -12) then ha_dawn=ha_dawn-24 
    if (ha_dusk lt !x.crange[0] and ha_dusk+24 lt  12) then ha_dusk=ha_dusk+24 

    if (ha_dawn lt !x.crange[0] and ha_dusk gt !x.crange[1]) then begin
        for q=0,n_elements(kk)-1 do begin
            oplot,[havec(kk(q)),havec(kk(q))],[!y.crange[0],!y.crange[1]]
        endfor
    endif

    if (ha_dusk gt !x.crange[1] and ha_dawn gt !x.crange[1]) then begin
        for q=0,n_elements(kk)-1 do begin
            oplot,[havec(kk(q)),havec(kk(q))],[!y.crange[0],!y.crange[1]]
        endfor
    endif

    if (ha_dusk lt !x.crange[0] and ha_dawn lt !x.crange[0]) then begin
        for q=0,n_elements(kk)-1 do begin
            oplot,[havec(kk(q)),havec(kk(q))],[!y.crange[0],!y.crange[1]]
        endfor
    endif

;=========================================================================================
; END Twilight Calculations
;========================================================================================
endif

end ; That's it!

;+
; NAME: sunpos
;
;
;
; PURPOSE: compute the position of the sun and return times (UT)
;          for astronomical twilight (both dusk and dawn) & LSMT times too.
;
;"Low precision formulas for the Sun's coordinates and the equation of time"
;From Astronomical Almanac (1989) page C24

;
; CALLING SEQUENCE:
;          sunpos,JD_IN,LAT,LONG,TW_DUSK,TW_DAWN,LMST_DUSK,LMST_DAWN
;
;
; INPUTS:
;          JD_IN = modified julian date
;
;          LAT = latitude in radians
;
;          LONG = longitude in radians
;
;          
; OPTIONAL INPUTS: none

;
;
; KEYWORD PARAMETERS: none
;
;
; OUTPUTS:

;          TW_DUSK = UT time of dusk (astronomical twilght) after sunset
;                    when zenith distance of the sun is at 120 degrees
;
;          TW_DAWN = UT time of dawn (astronomical twilght) before sunrise
;                    when zenith distance of the sun is at 120 degrees
;
;          LMST_DUSK = Local Mean Sideral Time of dusk 
;                    (astronomical twilght: after sunset when zenith
;                    distance of the sun is at 120 degrees)
;
;          LMST_DAWN = Local Mean Sideral Time of dawn
;                    (astronomical twilght: before sunrise when zenith
;                    distance of the sun is at 120 degrees);
;
;
; OPTIONAL OUTPUTS: none
;
;
;
; COMMON BLOCKS: none
;
;
;
; SIDE EFFECTS: none
;
;
;
; RESTRICTIONS: This will probably break if choose the LAT and LONG
;               near or above/below the artic/anarctic circles!
;
; EXAMPLE:
;
;     At CHARA,  25-DEC-1981 
;
;   sunpos,44963.5d0, ten(34.,13.,0.)*(!pi/180.d0),$
;        ten(-118.,03.,36.)*(!pi/180.d0),tw_dusk,tw_dawn,lmst_dusk,lmst_dawn
;
;   print,tw_dusk,tw_dawn
;   3.2904176       12.416673
;
;
; MODIFICATION HISTORY:
;
; comments added/improved: 13/feb/04 (jpa)
;
; first version early December 2003 (jpa)
;
;-

pro sunpos,jd_in,lat,long,tw_dusk,tw_dawn,lmst_dusk,lmst_dawn

n_points=96

;intialize vectors
altsun=fltarr(n_points) ;altitude of the sun
time=fltarr(n_points)   ;time in hours
stime=fltarr(n_points)  ;
ra_vec=fltarr(n_points)
ra_h_vec=fltarr(n_points)
dec_vec=fltarr(n_points)
ha_vec=fltarr(n_points)
lmst_vec=fltarr(n_points)
l_vec=fltarr(n_points)
g_vec=fltarr(n_points)
lambda_vec=fltarr(n_points)

for jj=0,n_points-1 do begin

    hour=jj/4.d0        ;hours run from 0 h to 23.75 h in 0.25 h steps

    jd=jd_in+0.5+(hour/24.d0)   ;compute MJD for each hour step

;from Almanac formulae
    n=jd-51545.d0

;Mean longitude of Sun, corrected for aberration
    L=280.460d0 + (0.9856474d0*n)

;Mean anomaly
    g=357.528d0 + (0.9856003d0*n)

;put L and g in the range 0 to 360 degrees by adding multiples of 360

    while (L gt 360.d0) do L=L-360.d0
    while (g gt 360.d0) do g=g-360.d0

;Ecliptic longitude
    lambda=L+(1.915d0 * sin(!pi*g/180.d0)) + (0.020 * sin (!pi*2.d0*g/180.d0))

    while (lambda lt 0.d0) do lambda=lambda+360.d0
    while (lambda gt 360.d0) do lambda=lambda-360.d0

;# Obliquity of ecliptic
    epsilon=23.439 - (0.0000004d0*n)

;# right ascension of the ran
    ra=atan(cos(epsilon*!pi/180.d0)*tan(lambda*!pi/180.d0))

    ra_deg=ra*180.d0/!pi

;# R.A. should be in the same quadrant as lambda, but the arctan
;# expression containing the obliquity of the ecliptic may place 
;# the R.A. 90 degrees off. adjust R.A. accordingly
;
;# We do this because the R.A. will be discontinuous across lambda
;# quadrants otherwise
;#

    if (lambda ge 0.d0 and lambda le 90.d0) then begin
        while (ra_deg lt 0.d0)  do ra_deg=ra_deg+90
        while (ra_deg gt 90.d0)  do ra_deg=ra_deg-90
    endif    

    if (lambda gt 90.d0 and lambda le 180.d0) then begin
        while (ra_deg lt 90.d0)  do ra_deg=ra_deg+90
        while (ra_deg gt 180.d0)  do ra_deg=ra_deg-90
    endif    

    if (lambda gt 180.d0 and lambda le 270.d0) then begin
        while (ra_deg lt 180.d0)  do ra_deg=ra_deg+90
        while (ra_deg gt 270.d0)  do ra_deg=ra_deg-90
    endif    

    if (lambda gt 270.d0 and lambda le 360.d0) then begin
        while (ra_deg lt 270.d0)  do ra_deg=ra_deg+90
        while (ra_deg gt 360.d0)  do ra_deg=ra_deg-90
    endif    

;#RA  of the sun in hours
    ra_h=ra_deg/15.d0

;# RA of the sun in radians
    ra=ra_h*15.d0*!pi/180.d0

;# Declination of the sun in radians
    dec=asin(sin(epsilon*!pi/180.d0)*sin(lambda*!pi/180.d0))

;# See  GMST (Page B6 from the Astronomical Almanac)
;# TIME-SCALES: "Relationships between universal and sidereal time"

;with reduced JD
    tu=(jd-51545.d0)/36525.d0 ;Julian centuries of 36525 days since ;J2000.0

;# GMST in seconds at 0h UT
    gmst= 24110.54841d0 + (8640184.812866d0*tu) + (0.093104d0*tu*tu) + (6.2d-6*tu*tu*tu)

;Greenwich mean sidereal time in hours
    gmst_h=(gmst/3600.d0)+ (hour*1.00273791d0)

    while(gmst_h gt 24) do gmst_h=gmst_h-24.d0 ;
    while(gmst_h lt 0) do gmst_h=gmst_h+24.d0 ;

;# LMST: local mean sidereal time in hours
    lmst_h=(gmst/3600.d0)+ (hour*1.00273791d0)+long*(180./!pi)/15.d0

;# H.A. of the sun
    ha_h=lmst_h-ra_h

;# H.A. in radians
    ha_rad=(ha_h*15.d0)*!pi/180.d0

;#
;# compute solar altitude
;#
    sin_altitude=sin(dec)*sin(lat)+cos(dec)*cos(ha_rad)*cos(lat)
    altitude=asin(sin_altitude)*180./!pi ;in degrees

;# save variables into vectors at each time step

    altsun(jj)=altitude
    time(jj)=hour
    stime(jj)=lmst_h
    ra_vec(jj) = ra
    ra_h_vec(jj) = ra_h
    dec_vec(jj) = dec*180.d0/!pi
    ha_vec(jj) = ha_h
    lmst_vec(jj) = lmst_h
    L_vec(jj) = L
    g_vec(jj) = g
    lambda_vec(jj) = lambda

endfor

;#
;# zenith distance of the sun in degrees
;#
zd=90.d0-altsun

;#
;# UT of astronomical twilight
;#

k=where(zd ge 108.d0) ; for all zd > 108 degrees = astronomical twilight
j=sort(zd(k))

if (zd(k(j(0))) lt zd(k(j(0))+1)) then begin
;zd is increasing with time so this is dusk
    tw_dusk=interpol(time(k(j(0))-1:k(j(0))),zd(k(j(0))-1:k(j(0))),108.d0)
endif else begin
    tw_dawn=interpol(time(k(j(0)):k(j(0))+1),zd(k(j(0)):k(j(0))+1),108.d0)
endelse

if (zd(k(j(1))) lt zd(k(j(1))+1)) then begin
;zd is increasing with time so this is dusk
    tw_dusk=interpol(time(k(j(1))-1:k(j(1))),zd(k(j(1))-1:k(j(1))),108.d0)
endif else begin
    tw_dawn=interpol(time(k(j(1)):k(j(1))+1),zd(k(j(1)):k(j(1))+1),108.d0)
endelse


;local mean sidereal time of dusk and dawn (in hours)
lmst_dusk=(gmst/3600.d0)+ (tw_dusk*1.00273791d0)+long*(180./!pi)/15.d0
lmst_dawn=(gmst/3600.d0)+ (tw_dawn*1.00273791d0)+long*(180./!pi)/15.d0

while(lmst_dawn gt 24) do lmst_dawn=lmst_dawn-24.d0 ;
while(lmst_dawn lt 0) do lmst_dawn=lmst_dawn+24.d0 ;

while(lmst_dusk gt 24) do lmst_dusk=lmst_dusk-24.d0 ;
while(lmst_dusk lt 0) do lmst_dusk=lmst_dusk+24.d0 ;


end ;That's it!

;@mykepler.pro
;+
; NAME: ORBIT2
;
;
;
; PURPOSE: Compute position angle and angular separation of the two components
;          of the binary star Alpha Virginis (Spica)
;
;
; CATEGORY:  visual binary orbital elements and computation
;
;
;
; CALLING SEQUENCE: orbit2,MJD,RHO,THETA
;
;
;
; INPUTS: MJD - Modified Julian Date (e.g. 45069.5)
;
;
;
; OPTIONAL INPUTS: none
;
;
;
; KEYWORD PARAMETERS: none
;
;
;
; OUTPUTS:  RHO - angular separation (in arcseconds)
;           THETA - position angle   (in radians)
;
;
; OPTIONAL OUTPUTS: none
;
;
;
; COMMON BLOCKS: none
;
;
;
; SIDE EFFECTS: none known 
;
;
; PROCEDURE:
;
;
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;
; added document header (jpa, 27/feb/04)
;
; first version late December 2003
;
;-
pro orbit2,mjd,rho,theta

;mjd : INPUT modified julian date (days)
;rho : OUTPUT apparent star separation (in arcSec)
;psi : OUTPUT angle between baseline and line connecting the two stars

date=mjd
;=============================================
;orbit params for Alf Vir  (1971 MNRAS 151 161)
;============================================sy
tzero=40678.09 ;mjd = jd -2400000
period=1./0.249091 ;days
kvel1=116.d0 ;km/s
kvel2=195.87 ;km/s from mass ratio
m1=10.9;+/- 0.8  msun
m2=6.8 ;+/- 0.7  msun
r1=8.1;+/- 0.9 rsun  
ecc=0.146    
omega=138.  ;degrees
inclin=65.9 ;+/- 1.8 degrees (1971 MNRAS 151 161)
gamma_1 = -5.8 ; +/- 1.3
gamma_2 = 8.8 ; +/- 8.8
k_1 = 123.9 ; +/- 1.4
k_2 = 198.8 ; +/- 1.5
mu=2*!pi/period
vsini = 155.d0 ;km/s
Paps = 124 ;years
theta_a=1.54e-3 ;angluar size of semi-major axis  in arc seconds (1971 MNRAS 151 161)
big_omega = 131.6 ;Longitude of the line of nodes (1971 MNRAS 151 161)

;===========================================

;# find true anomaly for this date
mykepler,date,tzero,mu,ecc,truea
ta=truea

;# advance longitude of periastron
yfrac=(date-tzero)/365.25d0
omega_rad=((omega+(360.*yfrac/Paps))*!pi/180.)

;# convert angles to radians
ta_rad=ta*!pi/180.
i_rad=inclin*!pi/180
big_omega_rad=big_omega*!pi/180.

sum=ta_rad+omega_rad

 while (sum lt 0.d0) do sum=sum+(2.d0*!pi)
 while (sum gt 2.d0*!pi) do sum=sum-(2.0*!pi)

;#
;# From Aikten, The Binary Stars (Dover): Chapter 4: Visual Binary Orbit
;# Equations (6) and (7) and (6a)


;#
;# theta is the position angle (east of north in radians)
;#
 if (sum ge !pi/2 and sum lt 3*!pi/2) then begin
     theta=big_omega_rad+atan(-tan(sum)*cos(i_rad)) ;position angle in radians
 endif

 if (sum lt !pi/2 or sum ge 3*!pi/2) then begin
     theta=big_omega_rad+atan(-tan(sum)*cos(i_rad)) ;position angle in radians
 endif

;#
;# little_r is the angular radius vector and rho is the separation in arcseconds
;#

  little_r=theta_a*(1.d0-(ecc^2))/(1.d0+(ecc*cos(ta_rad)))
  rho=little_r*cos(sum)/cos(theta-big_omega_rad) ;

;velocities of the two stars
;  vel_1=gamma_1+k_1*(ecc*cos(omega_rad)+cos(sum))
;  vel_2=gamma_2+k_2*(ecc*cos(!pi+omega_rad)+cos(!pi+sum))

;# that's it!
end

;+
; *NAME: mykepler 
;
; *PURPOSE:
; Computes the true anomaly (Solves Kepler's Equation) for an orbit given
;     (1) the present epoch
;     (2) epoch of periastron
;     (3) period
;     (4) eccentricity
;
; CALLING SEQUENCE: MYKEPLER,T,EPA,MU,ECC,TA
;
; INPUTS:
;        T   - epoch (date) in days
;        EPA - epoch of periastron in days
;        MU  - 2*pi/period  in days
;        ECC - eccentricity
;
;
; OUTPUTS:
;        E  - eccentric anomaly in degrees
;        TA - true anomaly in degrees
;
; EXAMPLE:
;
;
;
; MODIFICATION HISTORY:
;
; Documentation header, better comments (jpa,28/feb/04)
;
; More comments, testing December 2003 
;
; Orignal written in ~2001 by jpa 
;-

pro mykepler,t,epa,mu,ecc,ta

m=mu*(t-epa) ;Mean anomaly, a inital guess for eccentric anomaly
;#
;# if the eccentricity is zero, we're done
;#

if (ecc eq 0.0) then begin
ta=m*180.d0/!pi ;True anomaly
endif else begin

;iterative solution
;OFF;e=m          ;eccentric anomaly (that inital guess)
;OFF;for j=0,5 do begin
;OFF;e=e-(e-ecc*sin(e)-m)/(1.-ecc*cos(e))
;OFF;endfor

;
; from Aikten's Binary Stars
;
e=m+(ecc*sin(m)/(1.d0-ecc*cos(m))) ;eccentric anomoly

;true anomoly
const=sqrt((1.d0+ecc)/(1.d0-ecc))
ta=(2.d0*atan(const*tan(e/2.d0)))*180.d0/!pi ;true anomoly in degrees

endelse

;
; reduce to between 0 and 360 degrees
;
while (ta lt 0.d0) do ta=ta+360.d0
while (ta gt 360.d0) do ta=ta-360.d0

;
; convert eccentric anomoly to degrees
;
big_e=e*180.d0/!pi 
;
; reduce to between 0 and 360 degrees
;
while (big_e lt 0.d0) do big_e=big_e+360.d0
while (big_e gt 360.d0) do big_e=big_e-360.d0

;# that's it! 
end

;@iuegetopt
;@parcheck
;*****************************************************************************
;+                                                                  
;*NAME:
;
;    JULDATE
;
;*CLASS:
;
;    unit conversion
;
;*CATEGORY:
;
;*PURPOSE:                                   
;
;    Convert from calender to Reduced Julian Date
;
;*CALLING SEQUENCE:
;
;    JULDATE,/PROMPT           ; Prompt for Calender Date, print Julian Date
;    JULDATE,DATE,jd      
;
;*PARAMETERS:
;
;    DATE   (REQ) (I) (1) (I F)
;	    Array containing year,month (1-12),day, hour, minute & seconds,
;           all specified as numbers (Universal Time). Year after 1900
;           can be specified 2 ways, either for example, as 83 or 1983.
;           Years B.C should be entered as negative numbers.  If hour, minute
;           or seconds are not supplied, they will default to 0. 
;
;    JD     (REQ) (O) (0) (F)
;	    Reduced Julian Date, double precision scalar.  To convert to
;           Julian Date, add 2400000.   JULDATE will print the value of
;           JD at the terminal if less than 2 parameters are supplied, or if 
;           the /PROMPT keyword is set
;      
;    PROMPT (OPT) (I) (keyword)
;	    If this keyword is set and non-zero, then JULDATE will prompt
;           for the calender date at the terminal.
;
;*EXAMPLE:
;
;     A date of 25-DEC-1981 06:25 UT may be expressed as either
;
;     IDL> juldate,[81,12,25,6,25],jd       
;     IDL> juldate,[1981,12,25.2673611],jd 
;
;     In either case, one should obtain a reduced Julian Date of 
;     JD = 44963.7673611
;
;*SYSTEM VARIABLES USED:
;
;*INTERACTIVE INPUT:
;
;*SUBROUTINES CALLED:
;
;     PARCHECK
;     IUEGETOPT
;
;*FILES USED:
;
;*SIDE EFFECTS:
;
;*RESTRICTIONS:
;
;     Will not work for years between 0 and 99 A.D.  (since these are
;     interpreted as years 1900 - 1999).  Will not work for year 1582.
;
;*NOTES:
;
;*PROCEDURE:
;
;*MODIFICATION HISTORY:
;
;    Adapted from IUE RDAF (S. Parsons)   8-31-87
;    Algorithm from Sky and Telescope April 1981   
;    Added /PROMPT keyword, W. Landsman    September 1992
;    25 Nov 92 PJL added PARCHECK and RDAF prolog; changed GETOPT to IUEGETOPT
;    02 Nov 94 RWT allow optional seconds in input array
;
;-
;******************************************************************************
 pro juldate,date,jd,prompt=prompt
;
;;; on_error,2 
;
 npar = n_params(0)
 if ( (npar eq 0) and (not keyword_set(prompt)) ) then begin
    print,'JULDATE,DATE,jd     or      JULDATE,/prompt'
    print,'  DATE - 3-6 element vector containing [year,month,day,hour,min,sec]
    print,'  jd - reduced julian date (double precision)
    retall
 endif  ; npar eq 0
 parcheck,npar,[0,1,2],'JULDATE'
;
 if (n_elements(date) eq 0) then begin   
    opt = ''                                                          
    rd: read,' Enter Year,Month,Day,Hour,Minute,Seconds (All Numeric): ',opt
    iuegetopt,opt,date,'F'
 endif  ; n_elements(date) eq 0
;
 case n_elements(date) of      
    6:
    5: date = [ date, 0.]
    4: date = [ date, 0., 0.]    
    3: date = [ date, 0., 0., 0.]
    else: begin
;;;             message,'Illegal DATE Vector - must have a least 3 elements'
             print,'Illegal DATE Vector - must have a least 3 elements'
             print,'ACTION:  Returning.'
             retall
          end  ; else
 endcase  ; n_elements(date)
;
 iy = fix(date(0))                     
 im = fix(date(1))
 day = double(date(2)) + (date(3) + date(4)/60.0 + date(5)/3600.0) / 24.0
 if (iy lt 100) then iy = iy + 1900    
;
 if (im lt 3) then begin   ; If month is Jan or Feb, don't include leap day
    iy= iy - 1
    im = im + 12 
 endif  ; im lt 3
;
 a = fix(iy/100)
 ry = float(iy)
 if (iy lt 1582) then b = 0 else b = 2 - a + fix(a/4)     
 if (iy eq 1582) then message,'ERROR:  Year 1582 not covered.'   
;
 jd = fix(ry*0.25) + 365.*(ry -1860.) + fix(30.6001*(im+1.)) + b + day  - 105.5
;
 if ( (npar lt 2) or keyword_set(prompt) ) then begin      
    yr = fix(date(0))
    if (yr lt 100) then yr = yr + 1900
    print,form='(a,i4,a,i3,a,f9.5)',$ 
       ' Year ',yr,'    Month',fix(date (1) ),'    Day',day 
    print,form='(a,f15.5)',' Reduced Julian Date: ',jd                       
 endif  ; npar eq 2
;
 return
 end  ; juldate


;+
; Name:
;      	LEGEND 
; Purpose:
;	This procedure makes a legend for a plot.  The legend can contain
;	a mixture of symbols, linestyles, Hershey characters (vectorfont),
;	and filled polygons (usersym).
; Examples:
;	The call:
;		legend,['diamond','asterisk','square'],psym=[4,2,6]
;	produces:
;		-----------------
;		|		|
;		|  <> diamond	|
;		|  *  asterisk	|
;		|  [] square	|
;		|		|
;		-----------------
;	Each symbol is drawn with a plots command, so they look OK.
;	Other examples are given in usage.
; Usage:
;	legend,items,linestyle=linestyle	; vertical legend at upper left
;	legend,items,psym=psym			; ditto except using symbols
;	legend,items,psym=psym,/horizontal	; horizontal format
;	legend,items,psym=psym,box=0		; sans border
;	legend,items,psym=psym,delimiter='='	; embed an '=' betw psym & text
;	legend,items,psym=psym,margin=2		; 2-character margin
;	legend,items,psym=psym,position=pos	; position of legend
;	legend,items,psym=psym,number=2		; plot two symbols, not one
;	legend,items,/fill,psym=[8,8,8],colors=[10,20,30]; 3 filled squares
; Inputs:
;	items = text for the items in the legend, a string array.
;		You can omit items if you don't want any text labels.
;		For example, items = ['diamond','asterisk','square'].
; Optional Inputs:
;	linestyle = array of linestyle numbers  If linestyle(i) < 0, then omit 
;		ith symbol or line to allow a multi-line entry.
;	psym = array of plot symbol numbers.  If psym(i) is negative, then a
;		line connects pts for ith item.  If psym(i) = 8, then the
;		procedure usersym is called with vertices define in the 
;		keyword usersym.
;	N. B.: Choose either linestyle, psym, neither, or both.  If neither is
;		present, only the text is output.  If both linestyle and
;		psym parameters are present, they both have to have the
;		same number of elements, and normal plot behaviour occurs.
;		By default, if psym is positive, you get one point so there is
;		no connecting line.
;	vectorfont = vector-drawn characters for the sym/line column, e.g.,
;		['!9B!3','!9C!3','!9D!3'] produces an open square, a checkmark,
;		and a partial derivative, which might have accompanying items
;		['BOX','CHECK','PARTIAL DERIVATIVE'].  If vectorfont(i) = '',
;		then plots is called to make a symbol or a line, but if
;		vectorfont(i) is a non-null string, then xyouts is called.
;		There is no check that !p.font is set properly, e.g., -1 for
;		X and 0 for PostScript.  This can produce an error, e.g., use
;		!20 with PostScript and !p.font=0, but allows use of Hershey
;		*AND* PostScript fonts together.
; Optional Keywords:
;	/help = flag to print header
;	/horizontal = flag to make the legend horizontal
;	/vertical = flag to make the legend vertical (D=vertical)
;	box = flag to include/omit box around the legend (D=include)
;	delimiter = embedded character(s) between symbol and text (D=none)
;	colors = array of colors for plot symbols/lines (D=!color)
;	textcolors = array of colors for text (D=!color)
;	margin = margin around text measured in characters and lines
;	spacing = line spacing (D=bit more than character height)
;	pspacing = psym spacing (D=3 characters)
;	charsize = just like !p.charsize for plot labels
;	position = normalized coordinates of the upper left of the legend
;	number = number of plot symbols to plot or length of line (D=1)
;	usersym = 2-D array of vertices, cf. usersym in IDL manual. (D=square)
;	/fill = flag to fill the usersym
; Outputs:
;	legend to current plot device
; Optional Output Keywords:
;	corners = 4-element array, like !p.position, of the normalized
;	  coords for the box (even if box=0): [llx,lly,urx,ury].
;	  Useful for multi-column or multi-line legends, for example,
;	  to make a 2-column legend, you might do the following:
;	    c1_items = ['diamond','asterisk','square']
;	    c1_psym = [4,2,6]
;	    c2_items = ['solid','dashed','dotted']
;	    c2_line = [0,2,1]
;	    legend,c1_items,psym=c1_psym,corners=c1,box=0
;	    legend,c2_items,line=c2_line,corners=c2,box=0,pos=[c1(2),c1(3)]
;	    c = [c1(0)<c2(0),c1(1)<c2(1),c1(2)>c2(2),c1(3)>c2(3)]
;	    plots,[c(0),c(0),c(2),c(2),c(0)],[c(1),c(3),c(3),c(1),c(1)],/norm
;	  Useful also to place the legend.  Here's an automatic way to place
;	  the legend in the lower right corner.  The difficulty is that the
;	  legend's width is unknown until it's is plotted.  In this example,
;	  the legend is plotted twice: the first time in the upper left, the
;	  second time in the lower right.
;	    legend,['1','22','333','4444'],linestyle=indgen(4),corners=corners
;			; BOGUS LEGEND---FIRST TIME TO REPORT CORNERS
;	    xydims = [corners(2)-corners(0),corners(3)-corners(1)]
;			; SAVE WIDTH AND HEIGHT
;	    chdim=[!d.x_ch_size/float(!d.x_size),!d.y_ch_size/float(!d.y_size)]
;			; DIMENSIONS OF ONE CHARACTER IN NORMALIZED COORDS
;	    pos = [!x.window(1)-chdim(0)-xydims(0) $
;			,!y.window(0)+chdim(1)+xydims(1)]
;			; CALCULATE POSITION FOR LOWER RIGHT
;	    plot,findgen(10)	; SIMPLE PLOT; YOU DO WHATEVER YOU WANT HERE.
;	    legend,['1','22','333','4444'],linestyle=indgen(4),pos=pos
;			; REDO THE LEGEND IN LOWER RIGHT CORNER
;	  You can modify the pos calculation to place the legend where you
;	  want.  For example to place it in the upper right:
;	    pos = [!x.window(1)-chdim(0)-xydims(0),!y.window(1)-xydims(1)]
; Common blocks:
;	none
; Procedure:
;	If keyword help is set, call doc_library to print header.
;	See notes in the code.
; Restrictions:
;	Here are some things that aren't implemented.
;	- It would be nice to allow data and device coords as well.
;	- An orientation keyword would allow lines at angles in the legend.
;	- An array of usersyms would be nice---simple change.
;	- An order option to interchange symbols and text might be nice.
;	- Somebody might like double boxes, e.g., with box = 2.
;	- Another feature might be a continuous bar with ticks and text.
;	- There are no guards to avoid writing outside the plot area.
;	- There is no provision for multi-line text, e.g., '1st line!c2nd line'
;	  Sensing !c would be easy, but !c isn't implemented for PostScript.
;	  A better way might be to simply output the 2nd line as another item
;	  but without any accompanying symbol or linestyle.  A flag to omit
;	  the symbol and linestyle is linestyle(i) = -1.
; Side Effects:
; Modification history:
;	write, 24-25 Aug 92, F K Knight (knight@ll.mit.edu)
;	allow omission of items or omission of both psym and linestyle, add 
;	  corners keyword to facilitate multi-column legends, improve place-
;	  ment of symbols and text, add guards for unequal size, 26 Aug 92, FKK
;	add linestyle(i)=-1 to suppress a single symbol/line, 27 Aug 92, FKK
;	add keyword vectorfont to allow characters in the sym/line column,
;	  28 Aug 92, FKK
;-
pro legend,help=help,items,linestyle=linestyle,psym=psym,vectorfont=vectorfont $
  ,horizontal=horizontal,vertical=vertical,box=box,margin=margin $
  ,delimiter=delimiter,spacing=spacing,charsize=charsize,pspacing=pspacing $
  ,position=position,number=number,colors=colors,textcolors=textcolors $
  ,fill=fill,usersym=usersym,corners=corners
;
;	=====>> HELP
;
if keyword_set(help) then begin & doc_library,'legend' & return & endif
;
;	=====>> SET DEFAULTS FOR SYMBOLS, LINESTYLES, AND ITEMS.
;
ni = n_elements(items)
np = n_elements(psym)
nl = n_elements(linestyle)
nv = n_elements(vectorfont)
n = max([ni,np,nl,nv])					; NUMBER OF ENTRIES
strn = strtrim(n,2)					; FOR ERROR MESSAGES
if n eq 0 then message,'No inputs!  For help, type legend,/help.'
if ni eq 0 then begin
  items = replicate('',n)				; DEFAULT BLANK ARRAY
endif else begin
  szt = size(items)
  if (szt(szt(0)+1) ne 7) then message,'First parameter must be a string array.  For help, type legend,/help.'
  if ni ne n then message,'Must have number of items equal to '+strn
endelse
symline = (np ne 0) or (nl ne 0)			; FLAG TO PLOT SYM/LINE
if (np ne 0) and (np ne n) then message,'Must have 0 or '+strn+' elements in psym array.'
if (nl ne 0) and (nl ne n) then message,'Must have 0 or '+strn+' elements in linestyle array.'
if n_elements(linestyle) ne n then linestyle = intarr(n); D=SOLID
if n_elements(psym) ne n then psym = intarr(n)		; D=SOLID
if n_elements(vectorfont) ne n then vectorfont = replicate('',n)
;
;	=====>> CHOOSE VERTICAL OR HORIZONTAL ORIENTATION.
;
if n_elements(horizontal) eq 0 then begin 		; D=VERTICAL
  if n_elements(vertical) eq 0 then vertical = 1
endif else begin
  if n_elements(vertical) eq 0 then vertical = not horizontal
endelse
;
;	=====>> SET DEFAULTS FOR OTHER OPTIONS.
;
if n_elements(box) eq 0 then box = 1
if n_elements(margin) eq 0 then margin = 0.5
if n_elements(delimiter) eq 0 then delimiter = ''
if n_elements(charsize) eq 0 then charsize = !p.charsize
if charsize eq 0 then charsize = 1
if n_elements(spacing) eq 0 then spacing = 1.2
if n_elements(pspacing) eq 0 then pspacing = 3
xspacing = !d.x_ch_size/float(!d.x_size) * (spacing > charsize)
yspacing = !d.y_ch_size/float(!d.y_size) * (spacing > charsize)
if !x.window(0) eq !x.window(1) then begin
  plot,/nodata,xstyle=4,ystyle=4,[0],/noerase
endif
;	next line takes care of weirdness with small windows
pos = [min(!x.window),min(!y.window),max(!x.window),max(!y.window)]
if n_elements(position) eq 0 then position = [pos(0),pos(3)] + [xspacing,-yspacing]
if n_elements(number) eq 0 then number = 1
if n_elements(colors) eq 0 then colors = !color + intarr(n)
if n_elements(textcolors) eq 0 then textcolors = !color + intarr(n)
fill = keyword_set(fill)
if n_elements(usersym) eq 0 then usersym = 2*[[0,0],[0,1],[1,1],[1,0]]-1
;
;	=====>> INITIALIZE POSITIONS
;
yoff = 0.25*yspacing				; VERT. OFFSET FOR SYM/LINE.
maxx = 0					; SAVED WIDTH FOR DRAWING BOX
x0 = position(0) + (margin)*xspacing		; INITIAL X & Y POSITIONS
y0 = position(1) - (margin-0.5)*yspacing
y = y0						; STARTING X & Y POSITIONS
x = x0
if vertical then begin				; CALC OFFSET FOR TEXT START
  xt = 0					; DEFAULT X VALUE
  for i = 0,n-1 do begin
    if psym(i) eq 0 then num = (number + 1) > 3 else num = number
    if psym(i) lt 0 then num = number > 2	; TO SHOW CONNECTING LINE
    if psym(i) eq 0 then expand = 1 else expand = 2
    xt = (expand*pspacing*(num-1)*xspacing) > xt
    endfor
endif	; NOW xt IS AN X OFFSET TO ALIGN ALL TEXT ENTRIES.
;
;	=====>> OUTPUT TEXT FOR LEGEND, ITEM BY ITEM.
;	=====>> FOR EACH ITEM, PLACE SYM/LINE, THEN DELIMITER,
;	=====>> THEN TEXT---UPDATING X & Y POSITIONS EACH TIME.
;	=====>> THERE ARE A NUMBER OF EXCEPTIONS DONE WITH IF STATEMENTS.
;
for i = 0,n-1 do begin
  if vertical then x = x0 else y = y0		; RESET EITHER X OR Y
  x = x + xspacing				; UPDATE X & Y POSITIONS
  y = y - yspacing
  if (psym(i) eq 0) and (vectorfont(i) eq '') then num = (number + 1) > 3 else num = number
  if psym(i) lt 0 then num = number > 2		; TO SHOW CONNECTING LINE
  if psym(i) eq 0 then expand = 1 else expand = 2
  xp = x + expand*pspacing*indgen(num)*xspacing
  if (psym(i) gt 0) and (num eq 1) and vertical then xp = x + xt/2.
  yp = y + intarr(num)
  if vectorfont(i) eq '' then yp = yp + yoff
  if psym(i) eq 0 then begin
    xp = [min(xp),max(xp)]			; TO EXPOSE LINESTYLES
    yp = [min(yp),max(yp)]			; DITTO
    endif
  if psym(i) eq 8 then usersym,usersym,fill=fill,color=colors(i)
  if vectorfont(i) ne '' then begin
    if (num eq 1) and vertical then xp = x + xt/2
    xyouts,xp,yp,vectorfont(i),width=width,color=colors(i) $
      ,size=charsize,align=0.5,/norm
  endif else begin
    if symline and (linestyle(i) ge 0) then plots,xp,yp,color=colors(i) $
      ,/normal,linestyle=linestyle(i),psym=psym(i),symsize=charsize
  endelse
  if vertical then x = x + xt else x = max(xp)
  if symline then x = x + xspacing
  xyouts,x,y,delimiter,width=width,/norm,color=textcolors(i),size=charsize
  x = x + width
  if width gt 0 then x = x + 0.5*xspacing
  xyouts,x,y,items(i),width=width,/norm,color=textcolors(i),size=charsize
  x = x + width
  if not vertical and (i lt (n-1)) then x = x+2*xspacing; ADD INTER-ITEM SPACE
  maxx = (x + xspacing*margin) > maxx		; UPDATE MAXIMUM X
  endfor
;
;	=====>> OUTPUT BORDER
;
x = position(0)
y = position(1)
if vertical then bottom = n else bottom = 1
ywidth = - (2*margin+bottom-0.5)*yspacing
corners = [x,y+ywidth,maxx,y]
if box then plots,[x,maxx,maxx,x,x],y + [0,0,ywidth,ywidth,0],/norm
return
end
;
;
;
pro jplotopen,filename,eps=eps,ps=ps,color=color,port=port
;
; This is a simple routine to dump IDL plots in to PS or EPS files.
;

set_plot,'ps'
if (not keyword_set(port)) then begin
    if (not keyword_set(color)) then begin
        if (keyword_set(ps)) then device,/landscape,file=filename
        if (keyword_set(eps)) then device,/encapsulate,/landscape,file=filename
    endif else begin
        if (keyword_set(ps)) then device,/landscape,/color,file=filename
        if (keyword_set(eps)) then device,/encapsulate,/landscape,/color,file=filename
    endelse
endif else begin
    if (not keyword_set(color)) then begin
        if (keyword_set(ps)) then device,/portrait,file=filename
        if (keyword_set(eps)) then device,/portrait,/color,/encapsulate,file=filename
    endif else begin
        if (keyword_set(ps)) then device,/portrait,/color,file=filename
        if (keyword_set(eps)) then device,/portrait,/encapsulate,/color,file=filename
    endelse
endelse

end

pro jplotclose
;
; this just closes the plot device.  See jplotopen.pro (above)
;
device,/close
set_plot,'x'
end


